unit BUTTER; {unit X of SEASCAPE}

interface

  uses
    GLOBALS, UTILS, MATHS, LEGENDS, POST, SCREEN;

  procedure SumDifference (var theta: vector;
                  n: integer;
                  sign: integer);

  procedure FormMvec (var mVec: seasVector;
                  n: integer);

  procedure FormWvec (var wVec: seasVector;
                  n: integer;
                  lamOne, lamTwo: real);

  procedure QprimeY (var y: longVector;
                  Tcap: integer);

  procedure FormQy (var y: longVector;
                  Tcap: integer);

  procedure FormSVec (var sVec: seasVector;
                  n: integer);

  procedure FormTrend (var y: longVector;
                  var x: longVector;
                  lamOne: real;
                  Tcap: integer);

  procedure ButterworthFilter (n, Tcap: integer;
                  omegaCut: real;
                  var x: longVector;
                  var y: longVector);

  procedure MakeButterTrend (var dataBOX, trendBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var dataVector, trendVector, residueVec: longVector);

implementation

{_________________________________________________}

  procedure SumDifference (var theta: vector;
                  n: integer;
                  sign: integer);

 {This procedure finds the coefficients of the nth power of}
 {the summation operator or of the difference operator.}

    var
      j, q: integer;

  begin
    theta[0] := 1.0;
    for q := 1 to n do
      begin {q}
        theta[q] := 0.0;
        for j := q downto 1 do
          theta[j] := theta[j] + sign * theta[j - 1];
      end; {q}

  end; {SumDifference}

{_________________________________________________}

  procedure FormGamma (var gamma: seasVector;
                  mu: vector;
                  r: integer);

 {This procedure forms the Autcovariances  }
 {of an rth-order moving-average process.}

    var
      j, k: integer;

  begin
    for j := 0 to r do
      begin {j}
        gamma[j] := 0.0;
        for k := 0 to r - j do
          gamma[j] := gamma[j] + mu[k] * mu[k + j];
      end; {j}
  end; {FormGamma}

 {_________________________________________________}

  procedure FormMvec (var mVec: seasVector;
                  n: integer);

    var
      mu: vector;

  begin
    sumDifference(mu, n, 1);
    FormGamma(mVec, mu, n);
  end; {FormMvec}

 {_________________________________________________}

  procedure FormWvec (var wVec: seasVector;
                  n: integer;
                  lamOne, lamTwo: real);

    var
      i: integer;
      sVec, mVec: seasVector;

  begin
    FormSVec(sVec, n); {sVec = Q'SQ where Q is the 2nd operator}
    FormMVec(mVec, n);
    for i := 0 to n do
      wVec[i] := lamTwo * mVec[i] + lamOne * sVec[i];
  end; {FormwVec}

 {_________________________________________________}

  procedure QprimeY (var y: longVector;
                  Tcap: integer);

 {This procedure finds the second differences of the elements of a vector.}
 {Notice that the first two elements of the vector are lost in the process.}
 {However, we index the leading element of the differenced vector by t = 0.}

    var
      t: integer;

  begin
    for t := 0 to Tcap - 3 do
      y[t] := y[t] - 2 * y[t + 1] + y[t + 2];
  end; {QprimeY}

 {_________________________________________________}

  procedure FormQy (var y: longVector;
                  Tcap: integer);

 {This procedure multiplies the vector y of Tcap -2 elements by matrix}
 {Q of order Tcap times Tcap -2, where Q' is the matrix which finds}
 {the second differences of a vector.}

    var
      t: integer;
      lag1, lag2, store: real;

  begin
    lag1 := 0.0;
    lag2 := 0.0;
    for t := 0 to Tcap - 3 do
      begin {t}
        store := y[t];
        y[t] := y[t] - 2 * lag1 + lag2;
        lag2 := lag1;
        lag1 := store
      end; {t}
    y[Tcap - 2] := -2 * lag1 + lag2;
    y[Tcap - 1] := lag1;

  end; {FormQy}

 {_________________________________________________}

  procedure FormSVec (var sVec: seasVector;
                  n: integer);

{form  SQ. where is lambda?}

    var
      mu: vector;

  begin
    sumDifference(mu, n, -1);
    FormGamma(sVec, mu, n);
  end; {FormSvec}

 {_________________________________________________}

  procedure GammaY (gamma: seasVector;
                  var y: longVector;
                  Tcap, n: integer);

 {This procedure premultiplies a vector y by a symmetric banded Toeplitz matrix}
 {Gamma with n nonzero sub-diagonal bands and n nonzero supra-dagonal bands.}

    var
      i, t, j: integer;
      left, right: real;
      store: vector;

  begin
    for i := 0 to n do
      store[i] := 0.0;

    for t := 0 to Tcap - 1 do
      begin {t}
        for i := n downto 1 do
          store[i] := store[i - 1];
        store[0] := gamma[0] * y[t];
        for j := 1 to n do
          begin {j}
            if (t - j < 0) then
              left := 0
            else
              left := y[t - j];
            if (t + j > Tcap - 1) then
              right := 0
            else
              right := y[t + j];
            store[0] := store[0] + gamma[j] * (left + right);
          end; {j}
        if t >= n then
          y[t - n] := store[n];
      end; {t}
    for j := 0 to n - 1 do
      y[Tcap - 1 - j] := store[j];
  end; {GammaY}

 {_________________________________________________}

  procedure FormTrend (var y: longVector;
                  var x: longVector;
                  lamOne: real;
                  Tcap: integer);

    var
      t: integer;
      store: real;

  begin

    for t := 0 to Tcap - 1 do
      begin
        store := lamOne * x[t];
        x[t] := y[t] - store; {the trend vector}
        y[t] := store; {the residual vector}
      end;
  end; {FormTrend}

 {_________________________________________________}

  procedure ButterworthFilter (n, Tcap: integer;
                  omegaCut: real;
                  var x: longVector;
                  var y: longVector);

    var
      lamOne, lamVers: real;
      sVec, wVec: seasVector; {seasVector?}

  begin
    lamVers := Tan(omegaCut / 2);
    lamVers := Power(lamVers, 2 * n);
    LamOne := 1.0;

    x := y;
    FormWvec(wVec, n, lamOne, lamVers); { W = lamVers*M + Q'SQ }
    QprimeY(x, Tcap); {d = Q'y: we are loosing two points in taking 2nd differences}
    Toeplitz(wVec, x, Tcap - 3, n); {solve (lamVers*M + Q'SQ)g = d for g}
    FormQy(x, Tcap); {Form Qg}
    FormSvec(sVec, n - 2); {Form SQ}
    GammaY(sVec, x, Tcap, n - 2); {Form SQg}
    FormTrend(y, x, lamOne, Tcap); {x is trend, y is residual}

  end; {ButterworthFilter}

{_________________________________________________}

  procedure MakeButterTrend (var dataBOX, trendBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var dataVector, trendVector, residueVec: longVector);

    var
      minResidual, maxResidual, trendMean, residueMean: real;
      i: integer;
      x, y: longVector;

  begin
    for i := 0 to dataBOX.NOPUNCT - 1 do
      begin
        x[i] := dataVector[i];
        y[i] := dataVector[i];
      end;

    ButterworthFilter(filterSpec.filterOrder, dataBOX.NOPUNCT, filterSpec.cutOff, x, y);

  {MakeDataBox(residueBOX, residueVec); MakeDataBox(trendBOX, trendVec);}
    minResidual := y[0];
    maxResidual := y[0];
    trendMean := 0.0;
    residueMean := 0.0;
    for i := 0 to dataBOX.NOPUNCT - 1 do
      begin
        trendVector[i] := x[i];
        residueVec[i] := y[i];
        trendMean := trendMean + trendVector[i];
        residueMean := residueMean + residueVec[i];
        maxResidual := Rmax(maxResidual, y[i]);
        minResidual := Rmin(minResidual, y[i]);
      end;
    residueMean := residueMean / dataBOX.NOPUNCT;
    trendMean := trendMean / dataBOX.NOPUNCT;

    residueBOX := dataBOX;
    trendBOX := dataBOX;
    trendBOX.yMean := trendMean;
    residueBOX.yMean := residueMean;
    residueBOX.yMin := minResidual;
    residueBOX.yMax := maxResidual;

{weHaveEstimatedATrend := true;}
    weHaveTrendResiduals := true;

  end; {MakeButterTrend}

 {________________________________________________}

end. {BUTTERWORTH: Unit X of SEASCAPE.PAS}

