unit DATASEG; {Unit 8 of SEASIDE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, LEGENDS, POST, SCREEN, Dos, Crt, Graph;

  procedure GetTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PlotTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure LogOfVector (var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure AntiLogOfVector (var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure PolyRegress (var x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly, wgt: longVector;
                  q, n: integer);

  procedure ExePolyRegress (var x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly, wgt: longVector;
                  q, n: integer);

  procedure OrthoToPower (alpha, gamma, delta: vector;
                  var beta: vector;
                  q: integer);

  function PolyOrdinate (z: real;
                  var alpha, gamma, delta: vector;
                  q: integer): real;

  procedure MakePolyTrend (var polyParas: polyParameters;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec, PolyWgts: longVector);

  procedure QuadraticWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure CrenelatedWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure CosBellWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure UniformWgts (var PolyWgts: longVector;
                  var dataBOX: rBOX);

  procedure GetTheWeights (var PolyWgts: longVector;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure ExplainWeightedRegression;

  procedure FitPolyTrend (var polyParas: polyParameters;
                  var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, residueVec: longVector);

  procedure SlowFourier (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure MakeThePeriodogram (var dataBOX, spectraBOX: rBOX;
                  var spectraVector: longVector;
                  var alpha, beta: fourierVec);

  procedure PlotThePeriodogram (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, residueVec, adjustedVec, seasonVec: longVector);

implementation

  {________________________________________________}

    procedure GetTheData (var dataBOX: rBOX;
                    var SCREENBOX, POSTBOX: IBOX;
                    var dataVector: longVector);

      var
        theMenuItem: integer;

    begin
      repeat
        ClrScr;

        Writeln;
        Writeln('    GET THE DATA');
        Writeln;
        Writeln('         1. Read the Data');
        Writeln;
        Writeln('         2. Take Logarithms of the Data');
        Writeln;
        Writeln('         3. Extract a Polynomial Trend');
        Writeln;
        Writeln('         4. Plot the Data');
        Writeln;
        Writeln('         5. Plot the Periodogram of the Data and the Components');
        Writeln;
        Writeln('         6. Return to the Main Menu');
        Writeln;
        Writeln('                      Select a number and press RETURN');

        GetBoundedInteger(1, 6, theMenuItem);

        case theMenuItem of
          1:
            ReadTheData(dataBOX, dataVector); {in UTILS}
          2:
             begin
             if (logsTaken = false) then
               begin
                 LogOfVector(dataBOX, dataVector); {in DATASEG}
                 if (logsTaken = true) then
                   begin
                     Writeln;
                     Writeln('         The data have been replaced by their logarithms');
                     Readln;
                   end;
               end
             else
               begin
                 Writeln;
                 Writeln('         You have already taken logarithms of the data');
                 Readln;
               end;
            end;
          3:
            FitPolyTrend(polyParas, dataBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, residueVec);
          4:
            PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in DATASEG}
          5:
            PlotThePeriodogram(dataBOX, SCREENBOX, POSTBOX, dataVector, residueVec, adjustedVec, seasonVec);
   {in DATASEG}

        end;{case}

      until theMenuItem = 6;

    end;{GetTheData}

  {_________________________________________________}

   procedure PlotTheData (var dataBOX: rBOX;
                   var SCREENBOX, POSTBOX: IBOX;
                   var dataVector: longVector);
     var
       theMenuItem: integer;
       exitChar: char;
       exitNo: integer;
       legend: string;

   begin
     if (thereIsData = false) then
       begin
         SysBeep(3);
         {ReadTheData(dataBOX, dataVector);}
         Writeln('    First you must read some data!');
         Readln;
         GetTheData (dataBOX, SCREENBOX, POSTBOX, dataVector);
       end;

     if noPageParameters then
       begin
         SysBeep(3);
         PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
       end;

     theMenuItem := 1;
     exitChar := '2';
     exitNo := 2;

     repeat

       ClrScr;

       Writeln;
       Writeln('    PLOT THE DATA');
       Writeln;
       Writeln;
       Writeln('         1. Plot the Data Sequence');
       Writeln;

       if weHavePolyResiduals then
         begin
           Writeln('         2. Plot the Polynomial Residuals');
           Writeln;
           exitChar := '3';
           exitNo := 3;
           if weHaveSeasonalComponent then
             begin
               Writeln('         3. Plot the Seasonal Component');
               Writeln;
               Writeln('         4. Plot the Seasonaly Adjusted Data');
               Writeln;
               exitChar := '5';
               exitNo := 5;
             end;
         end;

       if weHaveButterworthResiduals then
         begin
           Writeln('         2. Plot the Trend-Cycle Function');
           Writeln;
           exitChar := '3';
           exitNo := 3;
           if weHaveSeasonalComponent then
             begin
               Writeln('         3. Plot the Seasonal Component');
               Writeln;
               Writeln('         4. Plot the Residue of the Threefold Decomposition');
               Writeln;
               exitChar := '5';
               exitNo := 5;
             end;
         end;

       Writeln('         ', exitChar, '. Return to the Main Menu');

       Writeln;
       Writeln('                      Select a number and press RETURN');

       GetBoundedInteger(1, exitNo, theMenuItem);

       if theMenuItem = 1 then
         begin
           legend := DataLegend(dataBOX, dataName);
           ScreenMakePlot(dataBOX, SCREENBOX, dataVector, legend);
           if (PSPict = true) then
             WritePostFile(dataBOX, POSTBOX, dataVector, legend);
         end;

       if (theMenuItem = 3) and (exitNo <> 3) then
         begin
           legend := SeasonalComponent(dataBOX, dataName);
           ScreenMakePlot(seasonBOX, SCREENBOX, seasonVec, legend);
           if (PSPict = true) then
             WritePostFile(seasonBOX, POSTBOX, seasonVec, legend);
         end;

       if weHavePolyResiduals then
         begin
           if theMenuItem = 2 then
             begin
               legend := PolynomialResiduals(polyParas.polyDegree, dataBOX, dataName);
               MakeDataBox(residueBOX, residueVec);
               ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);
               if (PSPict = true) then
                 WritePostFile(residueBOX, POSTBOX, residueVec, legend);
             end;

           if theMenuItem = 4 then
             begin
               legend := SeasonallyAdjustedData(dataBOX, dataName);
               ScreenMakePlot(dataBOX, SCREENBOX, adjustedVec, legend);
               if (PSPict = true) then
                 WritePostFile(seasonBOX, POSTBOX, seasonVec, legend);
             end;
         end; {weHavePolyResiduals}

       if weHaveButterworthResiduals then
         begin
           if theMenuItem = 2 then
             begin
               legend := ButterworthTrend(dataBOX, dataName, filterSpec);
               ScreenMakePlot(dataBOX, SCREENBOX, trendVector, legend);
               if (PSPict = true) then
                 WritePostFile(residueBOX, POSTBOX, residueVec, legend);
             end;

           if theMenuItem = 4 then
             begin
               legend := 'The residual vector of a threefold decomposition the data.';
               MakeDataBox(residueBOX, adjustedVec);
               ScreenMakePlot(residueBOX, SCREENBOX, adjustedVec, legend);
               if (PSPict = true) then
                 WritePostFile(residueBOX, POSTBOX, adjustedVec, legend);
             end;
         end;  {weHaveButterworthResiduals}

     until theMenuItem = exitNo;


   end; {PlotTheData}

{________________________________________________}

  procedure LogOfVector (var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      i: integer;
      yMax, yMin, yMean: real;
      BadElements: boolean;
      logDataVec: longVector;

  begin
    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
        {Writeln('    First you must read some data!');}
        {Readln;}
        {GetTheData (dataBOX, SCREENBOX, POSTBOX, dataVector);}
      end;

   if thereIsData then
   begin
    i := 0;
    yMean := 0.0;
    badElements := false;
    while (i < dataBOX.NOPUNCT) and (BadElements = false) do
      begin {while}
        if dataVector[i] <= 0 then
          begin
            Writeln('     The data contains elements with values that are zero or negative.');
            Writeln;
            Writeln('     The operation of taking logarithms of the data must be aborted');
            badElements := true;
            logsTaken := false;
            Writeln(' ' : 64, '<RETURN>');
            Readln;
          end
        else
          begin
            logDataVec[i] := ln(dataVector[i]);
            if i = 0 then
              begin
                yMax := logDataVec[0];
                yMin := yMax;
              end;
            yMean := yMean + logDataVec[i];
            if logDataVec[i] > yMax then
              yMax := logDataVec[i];
            if logDataVec[i] < yMin then
              yMin := logDataVec[i];
            i := i + 1;
            logsTaken := true;
          end;
      end; {while}
    yMean := yMean / dataBOX.NOPUNCT;

    if badElements = false then
      begin
        logsTaken := true;
        dataVector := logDataVec;
        dataBOX.yMax := yMax; {the maximum value of y in the Data vector}
        dataBOX.yMin := yMin; {the minimum value of y in the Data vector}
        dataBOX.yMean := yMean;
      end;
    end; {There is data}

  end; {LogOfVector}

{________________________________________________}

  procedure AntiLogOfVector (var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      i: integer;

  begin

    with dataBOX do
      begin
        yMax := Exp(dataVector[0]);
        yMin := yMax;
        yMean := 0.0;
        for i := 0 to NOPUNCT - 1 do
          begin {i}
            dataVector[i] := Exp(dataVector[i]);
            if dataVector[i] > yMax then
              yMax := dataVector[i];
            if dataVector[i] < yMin then
              yMin := dataVector[i];
            yMean := yMean + dataVector[i];
          end; {i}
        yMean := yMean / dataBOX.NOPUNCT;
      end; {with DataBOX}

    logsTaken := false;

  end; {AntiLogOfVector}

{_______________________________________________}

  procedure PolyRegress (var x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly, wgt: longVector;
                  q, n: integer);

    var
      phi, philag: longVector;
      denom, lagdenom, temp: real;
      k, t: integer;

  begin {PolyRegress}
    if (thereIsData = false) then
      begin
        SysBeep(3);
        {ReadTheData(dataBOX, dataVector);}
        Writeln('    First you must read some data!');
        Readln;
        GetTheData (dataBOX, SCREENBOX, POSTBOX, dataVector);
      end;

    alpha[0] := 0.0;
    gamma[1] := 0.0;
    denom := 0.0;

    for t := 0 to n do
      begin
        alpha[0] := alpha[0] + y[t] * wgt[t];
        gamma[1] := gamma[1] + x[t] * wgt[t];
        denom := denom + wgt[t];
      end;

    alpha[0] := alpha[0] / denom;
    gamma[1] := gamma[1] / denom;
    lagdenom := denom;

    for t := 0 to n do
      begin
        philag[t] := 1;
        phi[t] := x[t] - gamma[1];
        poly[t] := alpha[0];
      end;

    for k := 1 to q do
      begin {k}

        alpha[k] := 0.0;
        gamma[k + 1] := 0;
        denom := 0.0;

        for t := 0 to n do
          begin {t}
            alpha[k] := alpha[k] + y[t] * phi[t] * wgt[t];
            denom := denom + Sqr(phi[t]) * wgt[t];
            gamma[k + 1] := gamma[k + 1] + Sqr(phi[t]) * x[t] * wgt[t];
          end; {t}

        alpha[k] := alpha[k] / denom;
        gamma[k + 1] := gamma[k + 1] / denom;
        delta[k + 1] := denom / lagdenom;
        lagdenom := denom;

        for t := 0 to n do
          begin {t}
            poly[t] := poly[t] + alpha[k] * phi[t];
            temp := phi[t];
            phi[t] := (x[t] - gamma[k + 1]) * phi[t] - delta[k + 1] * philag[t];
            philag[t] := temp;
          end; {t}

      end; {k}

  end; {PolyRegress}

{________________________________________________}

  procedure ExePolyRegress (var x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly, wgt: longVector;
                  q, n: integer);

  {It looks  as if this is now redundant}

    var
      phi, philag: longVector;
      denom, lagdenom, temp: real;
      k, t: integer;

  begin {PolyRegress}

    alpha[0] := 0.0;
    gamma[1] := 0.0;
    denom := 0.0;

    for t := 0 to n do
      begin
        alpha[0] := alpha[0] + y[t] * wgt[t];
        gamma[1] := gamma[1] + x[t] * wgt[t];
        denom := denom + wgt[t];
      end;

    alpha[0] := alpha[0] / denom;
    gamma[1] := gamma[1] / denom;
    lagdenom := denom;

    for t := 0 to n do
      begin
        philag[t] := 1;
        phi[t] := x[t] - gamma[1];
        poly[t] := alpha[0];
      end;

    for k := 1 to q do
      begin {k}

        alpha[k] := 0.0;
        gamma[k + 1] := 0;
        denom := 0.0;

        for t := 0 to n do
          begin {t}
            alpha[k] := alpha[k] + y[t] * phi[t] * wgt[t];
            denom := denom + Sqr(phi[t]) * wgt[t];
            gamma[k + 1] := gamma[k + 1] + Sqr(phi[t]) * x[t] * wgt[t];
          end; {t}

        alpha[k] := alpha[k] / denom;
        gamma[k + 1] := gamma[k + 1] / denom;
        delta[k + 1] := denom / lagdenom;
        lagdenom := denom;

        for t := 0 to n do
          begin {t}
            poly[t] := poly[t] + alpha[k] * phi[t];
            temp := phi[t];
            phi[t] := (x[t] - gamma[k + 1]) * phi[t] - delta[k + 1] * philag[t];
            philag[t] := temp;
          end; {t}

      end; {k}

  end; {PolyRegress}

{________________________________________________}

  procedure OrthoToPower (alpha, gamma, delta: vector;
                  var beta: vector;
                  q: integer);

    var
      phiplus: real;
      phi, philag: vector;
      i, k: integer;

  begin
    phi[-1] := 0;
    phi[0] := -gamma[1];
    phi[1] := 1; {redundant}
    philag[0] := 1;
    philag[1] := 0;
    beta[0] := alpha[0];

  {find the power-form coefficients}
    for k := 1 to q do
      begin {k}
        beta[k] := 0.0;
        for i := k downto 0 do
          begin
            beta[i] := beta[i] + alpha[k] * phi[i];
            phiplus := phi[i - 1] - gamma[k + 1] * phi[i] - delta[k + 1] * philag[i];
            philag[i] := phi[i];
            phi[i] := phiplus;
          end;
        phi[k + 1] := 1;
        philag[K + 1] := 0;
      end;{k}

  end; {OrthoToPower}

{________________________________________________}

  function PolyOrdinate (z: real;
                  var alpha, gamma, delta: vector;
                  q: integer): real;

    {alpha, gamma, delta and q--> polyDegree should be subsumbed under polyParas}
{z is the absicissa, alpha[j] are parameters  of the power form of the polynomial}
    var
      d: vector;
      j: integer;

  begin
    d[q] := alpha[q];
    d[q - 1] := alpha[q - 1] + d[q] * (z - gamma[q]);
    for j := q - 2 downto 0 do
      d[j] := alpha[j] + d[j + 1] * (z - gamma[j + 1]) - d[j + 2] * delta[j + 2];
    PolyOrdinate := d[0];

  end; {PolyOrdinate}

 {________________________________________________}

  procedure MakePolyTrend (var polyParas: polyParameters;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec, PolyWgts: longVector);

 {Called by <FitPolyTrend> in DATASEG: alpha, gamma, delta and polyOrder--> polyDegree}
 {are subsumbed under polyParas}

    var
      x: longVector;
      i: integer;

  begin
    with PolyParas do
      begin {polyParas}
        tStart := 0;
        residueBOX := dataBOX;
        trendBOX := dataBOX;

        for i := 0 to dataBOX.NOPUNCT - 1 do
          x[i] := i;

        PolyRegress(x, dataVector, alpha, gamma, delta, trendVector, PolyWgts, polyDegree, dataBOX.NOPUNCT - 1);

        for i := 0 to dataBOX.NOPUNCT - 1 do
          residueVec[i] := dataVector[i] - trendVector[i];

      end; {polyParas}

    MakeDataBox(trendBOX, trendVector);
    MakeDataBox(residueBOX, residueVec);
    trendEstimated := true;
    weHaveTrendResiduals := true;
    weHavePolyResiduals := true;
    weHaveButterworthResiduals := false;


  end; {MakePolyTrend}

  {_______________________________________________}

  procedure QuadraticWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      answer: boolean;
      f0, f1, z1, z2, a, b, c: real;
      det, quadratic, midRange: real;
      t, Tcap, cutOff: integer;
      quadBOX: rBOX;
      legend: string;

  begin
    ClrScr;

    Writeln;
    Writeln('    QUADRATIC WEIGHTING FUNCTION');
    Writeln;
    Writeln('    The option exists for splitting the quadratic weighting function');
    Writeln('    by  interpolating a constant segment in  the middle that has the');
    Writeln('    minimum weighting');
    writeln;

    Tcap := dataBOX.NOPUNCT;
    YesorNo('   Do you wish to interpolate a middle segment?', answer);
    if answer then
      begin
        Writeln('    Specify the relative length of the middle range as a percentage');
        ReadBoundedNumber(0.0, 100.0, midRange);
        midRange := Midrange / 100.0;
        cutOff := Round(Tcap * (1.0 - MidRange) / 2.0);
        z1 := cutOff;
        z2 := 2 * cutOff;
      end
    else
      begin
        cutoff := Tcap div 2;
        z1 := (Tcap - 1) / 2.0;
        z2 := (Tcap - 1);
      end;

  {Find the quadratic parameters}

    f0 := maxWgt;
    f1 := minWgt;
    c := f0;
    det := z1 * (z1 * z2 - z2 * z2);
    det := 1.0 / det;
    c := f0;
    a := z2 * (f1 - f0) * det;
    b := -z2 * a;

    for t := 0 to Tcap div 2 do
      begin
        if (t <= cutoff) then
          begin
            quadratic := a * t + b;
            quadratic := quadratic * t + c;
          end
        else
          quadratic := minWgt;
        PolyWgts[t] := quadratic;
        PolyWgts[Tcap - 1 - t] := quadratic;
      end; {t}

    quadBOX := dataBOX;
    quadBOX.ymax := maxWgt;
    quadBOX.ymin := minWgt;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(quadBOX, SCREENBOX);
    ScreenPlot(quadBOX, SCREENBOX, PolyWgts);

    legend := 'The Quadratic Weighting Function';
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

  end; {QuadraticWgts}

  {_______________________________________________}

  procedure CrenelatedWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      answer: boolean;
      crenWeight, midRange: real;
      t, Tcap, cutOff: integer;
      crenBOX: rBOX;
      legend: string;

  begin
    ClrScr;

    Writeln;
    Writeln('    CRENELATED WEIGHTING FUNCTION');
    Writeln;
    Writeln('    By default, the first quarter of the sample has the maximum');
    Writeln('    weighting, the  middle half has the  minimum  weighting and');
    Writeln('    the final quarter has the maximum weighting');
    Writeln;

    Tcap := dataBOX.NOPUNCT;

    YesorNo('    Do you wish to vary the length of the middle range?', answer);
    if answer then
      begin
        Writeln('    Specify the relative length of the middle range as a percentage');
        ReadBoundedNumber(0.0, 100.0, midRange);
        midRange := Midrange / 100.0;
        cutOff := Round(Tcap * (1.0 - MidRange) / 2.0);
      end
    else
      cutoff := Tcap div 4;

    for t := 0 to Tcap div 2 do
      begin
        if (t <= cutoff) then
          begin
            crenWeight := MaxWgt;
          end
        else
          crenWeight := MinWgt;
        PolyWgts[t] := crenWeight;
        PolyWgts[Tcap - 1 - t] := crenWeight;
      end; {t}

    crenBOX := dataBOX;
    crenBOX.ymax := maxWgt;
    crenBOX.ymin := minWgt;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(crenBOX, SCREENBOX);
    ScreenPlot(crenBOX, SCREENBOX, PolyWgts);

    legend := 'The Crenelated Weighting Function';
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

  end; {CrenelatedWgts}

  {_______________________________________________}

  procedure CosBellWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      answer: boolean;
      pi, omega, gamma, cosWeight, midRange: real;
      t, Tcap, cutOff: integer;
      cosBOX: rBOX;
      legend: string;

  begin
    ClrScr;

    Writeln;
    Writeln('    COSINE-BELL WEIGHTING FUNCTION');
    Writeln;
    Writeln('    The option exists for  splitting the cosine-bell weighting');
    Writeln('    function by interpolating a constant segment in the middle');
    Writeln('    that has the minimum weighting');
    Writeln;

    pi := 4.0 * ArcTan(1.0);
    Tcap := dataBOX.NOPUNCT;
    YesorNo('   Do you wish to interpolate a middle segment?', answer);
    if answer then
      begin
        Writeln('    Specify the relative length of the middle range as a percentage');
        ReadBoundedNumber(0.0, 100.0, midRange);
        midRange := Midrange / 100.0;
        cutOff := Round(Tcap * (1.0 - MidRange) / 2.0);
      end
    else
      cutoff := Tcap div 2;

    for t := 0 to Tcap div 2 do
      begin
        if (t <= cutoff) then
          begin
            omega := (t * pi) / cutOff;
            gamma := (maxWgt - minWgt) / 2.0;
            cosWeight := minWgt + gamma * (1 + Cos(omega));
          end
        else
          cosWeight := minWgt;
        PolyWgts[t] := cosWeight;
        PolyWgts[Tcap - 1 - t] := cosWeight;
      end; {t}

    cosBOX := dataBOX;
    cosBOX.ymax := maxWgt;
    cosBOX.ymin := minWgt;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(cosBOX, SCREENBOX);
    ScreenPlot(cosBOX, SCREENBOX, PolyWgts);

    legend := 'The Cosine Bell Weighting Function';
     ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

  end; {CosBellWgts}

  {_______________________________________________}

  procedure UniformWgts (var PolyWgts: longVector;
                  var dataBOX: rBOX);

    var
      t, Tcap: integer;

  begin
    Tcap := dataBOX.NOPUNCT;
    for t := 0 to Tcap - 1 do
      PolyWgts[t] := 1.0;

  end; {UniformWgts}

  {_______________________________________________}

  procedure GetTheWeights (var PolyWgts: longVector;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

    var
      acceptance: boolean;
      theMenuItem: integer;
      minWgt, maxWgt: real;

  begin
    ClrScr;

    Writeln;
    Writeln('    SPECIFY THE WEIGHTING FUNCTION');
    Writeln;
    YesorNo('    Do you accept that the minimum weight should be unity', acceptance);
    if acceptance then
      minWgt := 1.0
    else
      begin
        Writeln('    Specify the minimum value for the weights');
        ReadBoundedNumber(0.0, 100.0, minWgt);
      end;
    Writeln('    Specify the maximum value for the weights');
    ReadBoundedNumber(minWgt, 100.0, maxWgt);

    ClrScr;

    Writeln;
    Writeln('    SPECIFY THE WEIGHTING FUNCTION');
    Writeln;
    Writeln('         1.  Quadratic Weighting Function');
    Writeln;
    Writeln('         2.  Crenelated Weighting Function');
    Writeln;
    Writeln('         3.  Cosine-Bell Weighting Function');
    Writeln;
    Writeln('         4.  Uniform Weighting Function');
    Writeln;
    Writeln('                      Select a number and press <RETURN>');

    GetBoundedInteger(1, 4, theMenuItem);
    case theMenuItem of
      1: 
        QuadraticWgts(PolyWgts, minWgt, maxWgt, dataBOX, SCREENBOX);
      2: 
        CrenelatedWgts(PolyWgts, minWgt, maxWgt, dataBOX, SCREENBOX);
      3: 
        CosBellWgts(PolyWgts, minWgt, maxWgt, dataBOX, SCREENBOX);
      4: 
        UniformWgts(PolyWgts, dataBOX);
    end; {case}

  end; {GetTheWeights}


  procedure ExplainWeightedRegression;

  begin
    ClrScr;

    Writeln;
    Writeln;
    Writeln('    WEIGHTED LEAST-SQUARES POLYNOMIAL REGRESSION');
    Writeln;
    Writeln('    The  objective is to ensure that  the fitted  polynomial will  pass');
    Writeln('    through the  midst of the scatters of  points  at either end of the');
    Writeln('    sample. This is to ensure that there are no radical disjunctions in');
    Writeln('    the  periodic extension of  the residual sequence where  the end of');
    Writeln('    one  replication of the  sequence  joins the beginning of the  next');
    Writeln('    replication. To  achieve this, the  polynomial is fitted by a least');
    Writeln('    squares regression, in which the maximum weight is given  to either');
    Writeln('    end of the sample and the minimum weight is at its mid point.');
    Writeln;
    Writeln(' ' : 63, '<RETURN>');
    Readln;

    ClrScr;

    Writeln;
    Writeln('    WEIGHTED LEAST-SQUARES POLYNOMIAL REGRESSION');
    Writeln;
    Writeln('    The user  is first asked  to specify the  maximum  and  the minimum');
    Writeln('    weights---the latter  being unity  by  default. They are also asked');
    Writeln('    to specify a U-shaped profile according  to  which the weights make');
    Writeln('    their transition from  the minimum to the maximum  values. Two such');
    Writeln('    choices are a  quadratic function  and a  crenelated  function that');
    Writeln('    jumps abruptly from the minimum value, which is maintained  over an');
    Writeln('    interval centred on the mid point, to  the maximum value, which, by');
    Writeln('    default, is the  value in  the first  and  the last quarters of the');
    Writeln('    sample.');
    Writeln;
    Writeln('    There is  also an  option  for  splitting  the quadratic  by inter-');
    Writeln('    polating  a  constant segment into  an interval centred on the  mid');
    Writeln('    point. The  equivalent option in respect of the crenelated function');
    Writeln('    is to allow  variations  in  the range in over which  the weighting');
    Writeln('    function assumes its minimum value');
    Writeln;
    Writeln(' ' : 63, '<RETURN>');
    Readln;

  end; {ExplainWeightedRegression}

  {_______________________________________________}

  procedure FitPolyTrend (var polyParas: polyParameters;
                  var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, residueVec: longVector);

{This is the calling procedure}
{tendBOX and trendVector can be made local to this procedure}
  {Called by MAIN}

  {This serves to plot the polynomial trend and the residual vector}
  {We need to put cycleVec and cycleBOX in the parameter list}
    var
      trendBOX: rBOX;
      trendVector, PolyWgts: longVector;
      legend: string;
      MenuChoice: integer;

  begin
    if noPageParameters then
      begin
        SysBeep(3);
        PageSetup(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}
      end;

    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector); {in UTILS}
      end;

    ClrScr;

    Writeln;
    Writeln('     FIT A POLYNOMIAL TREND');
    Writeln;
    Writeln('    1.   Fit by Ordinary Least-Squares Regression');
    Writeln;
    Writeln('    2.   Fit by Weighted Least-Squares Regression');
    Writeln;
    Writeln('                      Select a number and press <RETURN>');

    GetBoundedInteger(1, 2, menuChoice);
    case menuChoice of
      1: 
        UniformWgts(PolyWgts, dataBOX);
      2: 
        begin
          ExplainWeightedRegression;
          GettheWeights(PolyWgts, dataBOX, SCREENBOX); {in POLYNOME}
        end;
    end; {case}

    Writeln;
    Writeln('    What is the degree of the polynomial?');
    GetBoundedInteger(0, 12, polyParas.polyDegree);
  {_________This is the end of the interface section which might go into  <Organise>_____}

    trendBOX := dataBOX;
    trendBOX.xMin := 0;
    residueBOX := trendBOX;
    MakePolyTrend(polyParas, dataBOX, trendBox, residueBOX, dataVector, trendVector, residueVec, PolyWgts);

    if menuChoice = 1 then
      legend := PolynomialTrend(polyParas.polyDegree, dataBOX, dataName);
    if menuChoice = 2 then
      legend := PolyWeightTrend(polyParas.polyDegree, dataBOX, dataName);

    ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);

    if PSPict then
      WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);
    legend := PolynomialResiduals(polyParas.polyDegree, dataBOX, dataName);

{MakeDataBox(residueBOX, residueVec);}

    ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

    if PSPict then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend);

    ClrScr;

    dataAreTrended := true;
    weHaveTrendResiduals := true;
    weHavePolyResiduals := true;
    weHaveButterworthResiduals := false;

  end; {FitPolyTrend}

{________________________________________________}

  procedure SlowFourier (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var alpha: fourierVec;
                  var beta: fourierVec);
    var
      sign, j, n, t, Tcap: integer;
      pi, FourierInc, omegaj, omegajt: real;

  begin
    Tcap := dataBOX.NOPUNCT;
    n := Tcap div 2;

    pi := 4.0 * Arctan(1.0);
    FourierInc := 2 * pi / Tcap;

    alpha[0] := 0.0;
    beta[0] := 0.0;
    alpha[n] := 0.0;
    beta[n] := 0.0; {in case Tcap is even}

    for t := 0 to Tcap - 1 do
      begin
        alpha[0] := alpha[0] + dataVector[t];
      end;
    alpha[0] := alpha[0] / Tcap;

    if Even(Tcap) then
      begin
        sign := 1;
        for t := 0 to Tcap - 1 do
          begin
            alpha[n] := alpha[n] + sign * dataVector[t];
            sign := -sign;
          end;
        alpha[n] := alpha[n] / Tcap;
        n := n - 1;
      end;

    for j := 1 to n do
      begin {j}
        omegaj := FourierInc * j;
        alpha[j] := 0.0;
        beta[j] := 0.0;
        for t := 0 to Tcap - 1 do
          begin
            omegajt := omegaj * t;
            alpha[j] := alpha[j] + Cos(omegajt) * dataVector[t];
            beta[j] := beta[j] + Sin(omegajt) * dataVector[t];
          end;
        alpha[j] := (alpha[j] * 2.0) / Tcap;
        beta[j] := (beta[j] * 2.0) / Tcap;
      end; {j}

  end; {SlowFourier}

{________________________________________________}

  procedure MakeThePeriodogram (var dataBOX, spectraBOX: rBOX;
                  var spectraVector: longVector;
                  var alpha, beta: fourierVec);

    var
      j, Tcap, Ncap: integer;

  begin

    Tcap := dataBOX.NOPUNCT;
    Ncap := Tcap div 2;
    spectraBOX.NOPUNCT := Ncap + 1;
    spectraBOX.yMax := 0.0;
    spectraBOX.yMin := 0.0;

    spectraVector[0] :=  0.0; {this is tantamount to data mean adjustment}
    for j := 1 to Ncap do
      begin
        spectraVector[j] := alpha[j] * alpha[j] + beta[j] * beta[j];
        spectraVector[j] := (spectraVector[j] * Tcap) / 2;
        spectraBOX.yMax := RMax(spectraBOX.yMax, spectraVector[j]);
      end;

{observe that beta[0] = 0 and that, if T is even. then beta[Ncap] := 0}
{These values have been determined within <SlowFourier>}

  end; {MakeThePeriodogram }
  {_________________________________________________}

procedure PlotThePeriodogram (var dataBOX: rBOX;
                var SCREENBOX, POSTBOX: IBOX;
                var dataVector, residueVec, adjustedVec, seasonVec: longVector);

  var
    alpha, beta: fourierVec;
   {spectraBOX: rBOX;}
    theMenuIndex: integer;
    exitChar: char;
    exitNo: integer;
    Ncap, Tcap: integer;
    legend: string;
    F: text;

begin

  if (thereIsData = false) then
    begin
      SysBeep(3);
      ReadTheData(dataBOX, dataVector);
    end;


  if noPageParameters then
    begin
      SysBeep(3);
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
    end;

  Tcap := dataBOX.NOPUNCT;
  Ncap := Tcap div 2;

if  thereIsData and (noPageParameters = false) then
  repeat
    ClrScr;
    Writeln;
    Writeln('    PLOT THE PERIODOGRAMS');
    Writeln;
    Writeln('    The option  is provided of plotting  the peridogrogam of the original');
    Writeln('    data sequence  and plotting the periodograms of the derived sequences');
    Writeln('    which include the residual sequences obtained by detrending the data,');
    Writeln('    the seqeunce of  the seasonal fluctuations and the residual component');
    Writeln('    from a threefold  decoposition  of the data, which  also includes the');
    Writeln('    trend-cycle component and the seasonal component');

    Writeln;
    Writeln('         1. Plot the Periodogram of the Data');
    Writeln;
    exitChar := '2';
    exitNo := 2;

    if weHavePolyResiduals then
      begin
        Writeln('         2. Plot the Periodogram of the Polynomial Residuals');
        Writeln;
        exitChar := '3';
        exitNo := 3;
        if weHaveSeasonalComponent then
          begin
            Writeln('         3. Plot the Periodogram of the Seasonal Component');
            Writeln;
            exitChar := '4';
            exitNo := 4;
          end;
      end;

    if weHaveButterworthResiduals then
      begin
        Writeln('         2. Plot the Periodogram of the Butterworth Residuals');
        Writeln;
        exitChar := '3';
        exitNo := 3;

        if weHaveSeasonalComponent then
          begin
            Writeln('         3. Plot the the Periodogram of Seasonal Component');
            Writeln;
            Writeln('         4. Plot the the Periodogram of the Threefold Residue');
            Writeln;
            exitChar := '5';
            exitNo := 5;
          end;
      end;

    Writeln('         ', exitChar, '. Return to the Main Menu');

    Writeln;
    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, exitNo, theMenuIndex);
    spectraBOX.NOPUNCT := dataBOX.NOPUNCT;

    if theMenuIndex = 1 then
      begin
        spectraVec := dataVector;
        MakeDataBox(spectraBOX, dataVector);
        legend := 'The periogogram of the data sequence';
      end;

    if (theMenuIndex = 3) and (exitNo <> 3) then
      begin
        spectraVec := seasonVec;
        MakeDataBox(spectraBOX, seasonVec);
        legend := 'The periodogram of the seasonal fluctuations ';
      end;

    if (theMenuIndex = 2) and (exitNo <> 2) then
      begin
        spectraVec := residueVec;
        MakeDataBox(spectraBOX, residueVec);
        if weHavePolyResiduals then
          legend := 'The periodogram of the residual deviations from the polynomial trend';
        if weHaveButterworthResiduals then
          legend := 'The periodogram of the residual deviations from the Butterworth trend-cycle';
      end;

    if (theMenuIndex = 4) and (exitNo <> 4) then
      begin
        spectraVec := adjustedVec;
        MakeDataBox(spectraBOX, adjustedVec);
        legend := 'The periodigram of the residual vector of a threefold decomposition the data.';
      end;

    if (theMenuIndex <> ExitNo) then
      begin
        spectraBOX.NOPUNCT := dataBOX.NOPUNCT;
        SlowFourier(spectraBOX, spectraVec, alpha, beta); {in DATASEG}
        MakeThePeriodogram(dataBOX, spectraBOX, spectraVec, alpha, beta); {in DATASEG}

        spectraBOX.xMax := pi;
        spectraBOX.xMin := 0;
        spectraBOX.yUB := spectraBOX.yMax;
        spectraBOX.yLB := 0;
        spectraBOX.xLB := 0;
        spectraBOX.xRB := pi;

        GetGraphics(graphicsOn, SCREENBOX, currentMode);          ;

        SCREENBOX.xScale := false; {because we are using ZeroPiAxis}
        ScreenFrameLite(spectraBOX, SCREENBOX);

        if dataBOX.dataFrequency = Monthly then
          DoDecAxis(SCREENBOX)
        else
          ZeroPiAxis(SCREENBOX);

        spectraBOX.NOPUNCT := Ncap + 1;
        ScreenPlot(spectraBOX, SCREENBOX, spectraVec);
        ScreenCaption(legend);

        CallForReturn;
        Readln;
        CloseGraph;

        RestoreCrtMode;
        ClrScr;

        if PSPict then
          PostPeriogramPlot(F, spectraBOX, spectraVec, POSTBOX, legend);

      end; {theMenuIndex <> ExitNo}

  until theMenuIndex = exitNo;


end; {PlotThePeriodogram}

{________________________________________________}

end. {DATASEG: Unit 8 of SEASIDE.PAS}

