unit LEGENDS; {Unit 4 of SEASIDE.PAS}

interface

  uses
    GLOBALS, UTILS, Dos, Crt, Graph;

  function Sgn (number: real): char;

  procedure xScreenCaption (legend: string);

  function DataLegend (var dataBOX: rBOX;
                  var dataName: string): string;

  function DeSeasonalisedOverlay (var dataBOX: rBOX;
                  var dataName: string): string;

  function SmoothCycleOverlay (var dataBOX: rBOX;
                  var dataName: string): string;

  function SeasonallyAdjustedData (var dataBOX: rBOX;
                  var dataName: string): string;

  function SeasonalComponent (var dataBOX: rBOX;
                  var dataName: string): string;

  function TrendCycleResidue (var dataBOX: rBOX;
                 var dataName: string): string;

  function DataPeriodogram (var dataBOX: rBOX;
                  var dataName: string): string;

  function PolynomialWeights: string;

  function PolynomialTrend (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function PolyWeightTrend (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function PolynomialResiduals (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function ButterworthTrend (var dataBOX: rBOX;
                  var dataName: string;
                  var filterSpec: FilterRecord): string;

  function ButterworthResiduals (var dataBOX: rBOX;
                  var dataName: string;
                  var filterSpec: FilterRecord): string;

  function ButterworthOverlay: string;

  function TrigonometricalTrend (var degrees: real;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function TrendResiduals (var dataBOX: rBOX;
                  var dataName: string): string;

  function SeasonalFilterGain (var filterSpec: FilterRecord): string;

  function DoubleFilterGain (var filterSpec: FilterRecord): string;

  function TripleFilterGain (var filterSpec: FilterRecord): string;

  function GainOfButterworth (var filterSpec: FilterRecord): string;

  function FourierLowPass (var dataBOX: rBOX;
                  var dataName: string): string;

  function FourierBandPass (var dataBOX: rBOX;
                  var dataName: string): string;

  function TrendAndCycle (var dataBOX: rBOX;
                  var dataName: string): string;

  function SeasonallyAdjusted (var dataBOX: rBOX;
                  var dataName: string): string;

  function SpectralBand (var lowerCut, upperCut: real): string;


implementation

  {________________________________________________}

  function Sgn (number: real): char;
  begin
    if number >= 0 then
      Sgn := '+'
    else
      Sgn := ' ';
  end; {Sgn}

  {_________________________________________________}

  procedure xScreenCaption (legend: string);

    var
      i, q, width, distance, breakPoint, XS, XQ, YS: integer;
      firstString, secondString: string;

  begin
{XS := GetMaxX div 2;}
{YS := (GetMaxY) - 30;}

    width := Length(legend);

    if width > 65 then
      begin {Breakit }
        distance := width;
        for i := 1 to width do
          begin {i}
            if legend[i] = ' ' then
              begin {if legend}
                q := Abs(width div 2 - i);
                if distance > q then
                  begin {if distance}
                    breakPoint := i;
                    distance := q;
                  end; {if distance}
              end; {if legend}
          end; {i}

        firstString := Copy(legend, 1, breakPoint);
        secondString := Copy(legend, breakPoint + 1, width);
{width := TextWidth(firstString);}
        XQ := XS - (width div 2);
        YS := YS - 35;
        YS := SCREENBOX.Y2 + 60; {Alternative}
        MoveTo(XQ, YS);
{OutText(firstString);}
{width := TextWidth(secondString);}
        XQ := XS - (width div 2);
        YS := YS + 15;
        MoveTo(XQ, YS);
{OutText(secondString);}
      end {Breakit}
    else
      begin
{width := TextWidth(legend);}
        XS := XS - (width div 2);
        YS := YS - 30;
        YS := SCREENBOX.Y2 + 60; {Alternative}
        MoveTo(XS, YS);
{OutText(legend);}
      end;

  end; {ScreenCaption}

  {________________________________________________}

  function DataLegend (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The plot of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ');
    legend := Concat(legend, dataName);
    DataLegend := legend;
  end; {DataLegend}

{________________________________________________}

  function DeSeasonalisedOverlay (var dataBOX: rBOX;
                  var dataName: string): string;
    var
      legend, addString: string;

  begin
    legend := 'The plot of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString);
    if (dataBOX.dataFrequency = Monthly) then
      addString := ' monthly ';
    if (dataBOX.dataFrequency = Quarterly) then
      addString := ' quarterly ';
    legend := Concat(legend, addstring, 'observations on ');
    legend := Concat(legend, dataName);
    addString := ' with a superimposed seasonally-adjusted sequence';
    legend := Concat(legend, addstring);
    If weHaveButterworthResiduals then
      begin
        addString := ', which is a component of a threefold decomposition of the data.';
        legend := Concat(legend,addString);
      end;
    DeSeasonalisedOverlay := legend;
  end; {DeSeasonalisedOverlay}

{________________________________________________}

function SmoothCycleOverlay (var dataBOX: rBOX;
                var dataName: string): string;
  var
    legend, addString: string;

begin
  legend := 'The plot of ';
  addString := IntegerToString(dataBOX.NOPUNCT);
  legend := Concat(legend, addString);
  if (dataBOX.dataFrequency = Monthly) then
    addString := ' monthly ';
  if (dataBOX.dataFrequency = Quarterly) then
    addString := ' quarterly ';
  legend := Concat(legend, addstring, 'observations on ');
  legend := Concat(legend, dataName);
  addString := ' with a superimposed smoothed cycle.';
  legend := Concat(legend, addstring);
  SmoothCycleOverlay := legend;
end; {SmoothCycleOverlay}

{________________________________________________}

  function SeasonallyAdjustedData (var dataBOX: rBOX;
                  var dataName: string): string;
    var
      legend, addString: string;

  begin
    legend := 'The plot of a seasonally adjusted sequence of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString);
    if (dataBOX.dataFrequency = Monthly) then
      addString := ' monthly ';
    if (dataBOX.dataFrequency = Quarterly) then
      addString := ' quarterly ';
    legend := Concat(legend, addstring, 'observations on ');
    legend := Concat(legend, dataName);
    SeasonallyAdjustedData := legend;
  end; {SeasonallyAdjustedData}

{________________________________________________}

  function SeasonalComponent (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The seasonal component extracted from ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ');
    legend := Concat(legend, dataName);
    If weHaveButterworthResiduals then
      begin
        addString := ', which is a component of a threefold decomposition of the data.';
        legend := Concat(legend,addString);
      end;
    SeasonalComponent := legend;

  end; {SeasonalComponent}

 {________________________________________________}

 function TrendCycleResidue (var dataBOX: rBOX;
                 var dataName: string): string;

   var
     legend, addString: string;

 begin
   legend := 'The residual sequence after the trend-cycle has been extracted from ';
   addString := IntegerToString(dataBOX.NOPUNCT);
   legend := Concat(legend, addString, ' points of ');
   legend := Concat(legend, dataName);
   TrendCycleResidue := legend;

 end; {TrendCycleResidue}

 {________________________________________________}

  function DataPeriodogram (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The periodogram of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    DataPeriodogram := legend;
  end; {DataPeridogram}

  {________________________________________________}

  function PolynomialWeights: string;

    var
      legend: string;
  begin
    legend := 'The profile of the weights applied to the data points ';
    PolynomialWeights := Concat(legend, 'in the polynomial regression');
  end; {PolynomialWeights}

  {________________________________________________}

  function PolynomialTrend (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A polynomial of degree ';
    addString := IntegerToString(polyOrder);
    legend := Concat(legend, addString, ' interpolated through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    PolynomialTrend := legend;
  end; {PolynomialTrend}

  {________________________________________________}

  function PolyWeightTrend (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A polynomial of degree ';
    addString := IntegerToString(polyOrder);
    legend := Concat(legend, addString, ' interpolated by weighted ');
    legend := Concat(legend, 'least-squares regression through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    PolyWeightTrend := legend;
  end; {PolynomialTrend}

  {________________________________________________}

  function PolynomialResiduals (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The residual sequence from fitting a polynomial of degree ';
    addString := IntegerToString(polyOrder);
    legend := Concat(legend, addString, ' to ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    PolynomialResiduals := legend;
  end; {PolynomialResiduals}

  {________________________________________________}

    function ButterworthTrend (var dataBOX: rBOX;
                    var dataName: string;
                    var filterSpec: FilterRecord): string;

      var
        legend, addString: string;
    {pi, degrees: real;}

    begin
    {pi := 4.0 * Arctan(1.0);}
      legend := 'The trend estimated by a Butterworth lowpass filter of order ';
      addString := IntegerToString(filterSpec.filterOrder);
      legend := Concat(legend, addString, ' interpolated through ');
      addString := IntegerToString(dataBOX.NOPUNCT);
      legend := Concat(legend, addString, ' points of ', dataName);

    {legend := Concat(legend, addString, ', with a nominal cut-off point at ');}
    {degrees := 180.0 / pi;}
    {degrees := degrees * filterSpec.cutOff;}
    {addString := IntegerToString(Round(degrees));}
    {legend := Concat(legend, addString, ' degrees');}
      ButterworthTrend := legend;

    end; {ButterworthTrend}

    {________________________________________________}

    function ButterworthResiduals (var dataBOX: rBOX;
                    var dataName: string;
                    var filterSpec: FilterRecord): string;

      var
        legend, addString: string;
    {pi, degrees: real;}

    begin
    {pi := 4.0 * Arctan(1.0);}
      legend := 'The residual sequence from detrending ';
      addString := IntegerToString(dataBOX.NOPUNCT);
      legend := Concat(legend, addString, ' points of ', dataName);
      legend := Concat(legend, ' via a Butterworth lowpass filter of order ');
      addString := IntegerToString(filterSpec.filterOrder);
      legend := Concat(legend, addString);
    {legend := Concat(legend, addString, ', with a nominal cut-off point at ');}
    {degrees := 180.0 / pi;}
    {degrees := degrees * filterSpec.cutOff;}
    {addString := IntegerToString(Round(degrees));}
    {legend := Concat(legend, addString, ' degrees');}
      ButterworthResiduals := legend;

    end; {ButterworthResiduals}

 {________________________________________________}

 function ButterworthOverlay: string;

 var
   legend, addString: string;

   begin
    legend := 'The residual sequence from de-tending via a Butterworth filter with';
    addstring :=  ' a superimposed seasonally-adjusted version, which is the residual';
    legend := Concat(legend, addString);
    addstring :=  ' sequence in a threefold decomposition of the data.';
    legend := Concat(legend, addString);
    ButterworthOverlay := legend;

   end;  {ButterworthOverlay}

 {________________________________________________}

  function VariSmoothTrend (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString, frequency: string;

  begin
    legend := 'A trend determined by a filter with a variable ';
    addString := 'smoothing parameter applied to ';
    legend := Concat(legend, addString);
    addString := IntegerToString(dataBOX.NOPUNCT);
    if dataBOX.dataFrequency = monthly then
      frequency := ' monthly ';
    if dataBOX.dataFrequency = Quarterly then
      frequency := ' quarterly ';
    if dataBOX.dataFrequency = Annual then
      frequency := ' annual ';
    legend := Concat(legend, addString, frequency, 'points of ', dataName);
    VariSmoothTrend := legend;

  end; {VariSmoothTrend}

  {________________________________________________}

  function VariSmoothResiduals (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The residual sequence from fitting a trend via a ';
    addString := 'filter with a variable smoothing parameter to ';
    legend := Concat(legend, addString);
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    VariSmoothResiduals := legend;
  end; {VariSmoothResiduals}

  {________________________________________________}

  function TrigonometricalTrend (var degrees: real;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A trend comprising Fourier components of frequencies less than ';
    addString := TalyToString(degrees);
    legend := Concat(legend, addString, ' degrees per period interpolated through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    TrigonometricalTrend := legend;

  end; {TrigonometricalTrend}

  {________________________________________________}

  function TrendResiduals (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The residual sequence from detrending ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName, ' via the Fourier method');
    TrendResiduals := legend;
  end; {TrendResiduals}


{________________________________________________}

  function SeasonalFilterGain (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;

  begin
    legend := 'The frequency response function of a time-domain filter ';
    addString := 'for the seasonal adjustment of a ';
    legend := Concat(legend, addString);
    if (filterSpec.seasonality = Monthly) then
      addString := ' monthly ';
    if (filterSpec.seasonality = Quarterly) then
      addString := ' quarterly ';
    legend := Concat(legend, addString, 'data sequence');
    if weHavePolyResiduals then
       begin
       addString := ', superimposed on the periodogram of the polynomial residuals';
        legend := Concat(legend, addstring);
       end;
    if weHaveButterworthResiduals then
       begin
       addString := ', superimposed on the periodogram of the Butterworth residuals.';
        legend := Concat(legend, addstring);
       end;
    SeasonalFilterGain := legend;
  end; {SeasonalFilterGain}

  {________________________________________________}

  function DoubleFilterGain (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;

  begin
    legend := 'The frequency response function of a double time-domain filter for ';
    if (filterSpec.seasonality = Monthly) then
      addString := ' monthly ';
    if (filterSpec.seasonality = Quarterly) then
      addString := ' quarterly ';
    legend := Concat(legend, addString, 'seasonal adjustment');
     if weHavePolyResiduals then
       begin
       addString := ', superimposed on the periodogram of the polynomial residuals.';
        legend := Concat(legend, addstring);
       end;
    if weHaveButterworthResiduals then
       begin
       addString := ', superimposed on the periodogram of the Butterworth residuals.';
        legend := Concat(legend, addstring);
       end;
       addString := ' The zeros are at either side of the seasonal frequencies.';
    legend := Concat(legend, addString);
    DoubleFilterGain := legend;

  end; {DoubleFilterGain}

  {________________________________________________}

  function TripleFilterGain (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;

  begin
    legend := 'The frequency response function of a triple filter for ';
    if (filterSpec.seasonality = Monthly) then
      addString := 'monthly ';
    if (filterSpec.seasonality = Quarterly) then
      addString := 'quarterly ';
    legend := Concat(legend, addString, 'seasonal adjustment.');
    addString := ' The zeros are at the seasonal frequencies and at either side.';
    legend := Concat(legend, addString);
    if weHavePolyResiduals then
       begin
       addString := ', superimposed on the periodogram of the polynomial residuals.';
        legend := Concat(legend, addstring);
       end;
    if weHaveButterworthResiduals then
       begin
       addString := ', superimposed on the periodogram of the Butterworth residuals.';
        legend := Concat(legend, addstring);
       end;
    TripleFilterGain := legend;

  end; {TripleFilterGain}

  {________________________________________________}

   function GainOfButterworth (var filterSpec: FilterRecord): string;

     var
       legend, addString: string;
       pi, degrees: real;

   begin
     pi := 4.0 * Arctan(1.0);
     degrees := (filterSpec.cutOff * 180)/pi;
     legend := 'The frequency response of the Butterworth lowpass';
     legend := Concat(legend, ' filter of order ');
     addString := IntegerToString(filterSpec.filterOrder);
     legend := Concat(legend, addString, ', with a nominal cut-off point at ');
     addString := TalyToString(degrees);
     legend := Concat(legend, addString, ' degrees');
     if weHavePolyResiduals then
       begin
       addString := ', superimposed on the periodogram of the polynomial residuals.';
        legend := Concat(legend, addstring);
       end;
     GainOfButterworth := legend;

   end; {GainOfButterworth}

   {________________________________________________}

  function FourierLowPass (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A low-frequency trend/cycle component extracted from ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName, ' by a Fourier Method');
    FourierLowPass := legend;

  end; {FourierLowPass}

  {________________________________________________}

  function FourierBandPass (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'Cyclical fluctations extracted from ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName, ' by a Fourier Bandpass Method');
    FourierBandPass := legend;

  end; {FourierBandPass}

  {________________________________________________}

  function TrendAndCycle (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The trend-cycle component derived by adding the interpolated ';
    addString := 'polynomial to the low-frequency components of the residual sequence';
    legend := Concat(legend, addString);
    TrendAndCycle := legend;

  end; {TrendAndCycle}

 {________________________________________________}

  function SeasonallyAdjusted (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A seasonally adjusted version of a sequence of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ');
    legend := Concat(legend, dataName);
    SeasonallyAdjusted := legend;

  end; {SeasonallyAdjusted}

 {________________________________________________}

  function SpectralBand (var lowerCut, upperCut: real): string;

    var
      legend, addString: string;

  begin
    legend := 'A spectral band, with a lower bound of ';
    addString := TalyToString(lowerCut);
    legend := Concat(legend, addString, ' degrees and an upper bound of ');
    addString := TalyToString(upperCut);
    legend := Concat(legend, addString, ' degrees, masking the periodogram');
    SpectralBand := legend;
  end; {SpectralBand}

  {________________________________________________}


end. {LEGENDS: Unit 4 of SEASIDE.PAS}

