unit ORGANISE; {Unit 11 of SEASIDE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, LEGENDS, SCREEN, POST, DATASEG, BUTTER, FILTER, RESPONSE, Dos, Crt, Graph;

  procedure FilterFrequency (var filterSpec: FilterRecord;
                  var dataBOX: rBOX);

  procedure SpecifyTheFilter (var filterSpec: filterRecord;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var rhoVec, phi, phi1, phi2, theta, theta1, theta2: seasVector);

  procedure DetrendTheData (var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, residueVec: longVector);

  procedure SaveComponents (var dataBOX, residueBOX, seasonBOX: rBOX;
                  var dataVector, residueVec, seasonVec, adjustedVec: longVector);

  procedure SaveAndExit (var dataBOX, residueBOX, seasonBOX: rBOX;
                  var dataVector, residueVec, seasonVec, adjustedVec: longVector);

  procedure TransformTheData (var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, resideVec: longVector;
                  var polyParas: polyParameters);

  procedure DoSeasonalAdjustment (var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, resideVec: longVector;
                  var polyParas: polyParameters);

  procedure ApplyTheFilters (var filterSpec: filterRecord;
                  var phi, phi1, phi2, theta, theta1, theta2: seasVector;
                  var spectraBOX, dataBOX, residueBOX: rBOX;
                  var spectraVec, dataVector, residueVec, seasonVec: longvector);

  procedure ApplyButterFilter (var dataBOX, trendBOX, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector;
                  var filterSpec: FilterRecord);


implementation

{_________________________________________________}

  procedure FilterFrequency (var filterSpec: FilterRecord;
                  var dataBOX: rBOX);

    var
      acceptIt, getSeasonality: boolean;
      dataChar: char;

  begin

    if thereisData = true then
      begin
        acceptIt := false;
        getSeasonality := false;

        if dataBOX.dataFrequency = Monthly then
          Writeln('   The data is observed with a Monthly frequency.');
        if dataBOX.dataFrequency = Quarterly then
          Writeln('   The data is observed with a Quarterly frequency.');

        if dataBOX.dataFrequency in [Monthly, Quarterly] then
          begin
            Writeln;
            Writeln('   Do you wish to equate the seasonal-adjustment frequency');
            YesOrNo('   with the data frequency?', acceptIt);
          end;
        if acceptIt then
          filterSpec.seasonality := dataBOX.dataFrequency
        else
          getSeasonality := true;
      end;

    if (thereisData = false) or (getSeasonality = true) then
      repeat
        Writeln('   Do you wish to make Quaterly or Monthly seasonal adjustment? Type Q or M');
        Readln(dataChar);
        if dataChar in ['Q', 'q'] then
          filterSpec.seasonality := Quarterly;
        if dataChar in ['M', 'm'] then
          filterSpec.seasonality := Monthly;
      until dataChar in ['Q', 'q', 'M', 'm'];

    noDataFrequency := false;
    filterSpec.s := filterSpec.seasonality

  end; {FilterFrequency}

{_________________________________________________}

  procedure SpecifyTheFilter (var filterSpec: filterRecord;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var rhoVec, phi, phi1, phi2, theta, theta1, theta2: seasVector);

{we should include SpectraBOx spectraVector amongst the parameters}
{whatabput SCREENBOX and POSTBOX?}

    var
      theMenuItem: integer;
      addString, legend: string;
      reSpecify: boolean;

  begin
    reSpecify := true;
    if noSeaFilterSpec = false then
      begin
        respecify := false;
        if filterSpec.filterType = singleFilter then
          addString := 'single filter';
        if  filterSpec.filterType = TaperedFilter then
          addString := 'trend-cycle filter';
        if  filterSpec.filterType = doubleFilter then
          addString := 'double filter';
        if  filterSpec.filterType = tripleFilter then
          addString := 'triple filter';
        legend := concat('         A ', addString, ' has already been specified');
        Writeln(legend);
        YesOrNo('         Do you wish to respecify the filter?', reSpecify);
      end;

    if reSpecify then
      begin {reSpecify}
        weHaveSeasonalComponent := false;
        weHaveTrendComponent := false;
        weHaveAdjustedResiduals := false;

        if noPageParameters then
          begin
            SysBeep(3);
            PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
          end;

        ClrScr;

        Writeln;
        Writeln('   SPECIFY THE SEASONAL-ADJUSTMENT FILTER');
        Writeln;
        Writeln('   There are three alternative filters that may be specified:');
        Writeln;
        Writeln('   (1) The ordinary filter  places its  poles  and  zeros at the fundamental');
        Writeln('       seasonal frequency and at its harmonics. The filter can be compounded');
        Writeln('       with a smoothing filter in order to  estimate a trend-cycle function.');
        Writeln;
        Writeln('   (2) The double offset  filter places  the poles  and zeros on either side');
        Writeln('       of these frequencies at a  distance specified in degrees that applies');
        Writeln('       uniformly to all of the frequencies.');
        Writeln;
        Writeln('   (3) The triple filter allows the  offsets to be specified for each of the');
        Writeln('       frequencies, and it incoporates the ordinary filter, which eliminates');
        Writeln('       the elements at the seasonal frequency and its harmonics.');
        Writeln;
        Writeln(' ' : 68, '<RETURN>');
        Readln;
        Writeln;

        FilterFrequency(filterSpec, dataBOX);

        repeat
          ClrScr;

          Writeln;
          Writeln('   SPECIFY THE SEASONAL-ADJUSTMENT FILTER');
          Writeln;
          Writeln('         1. The Basic Seasonal-Adjustment Filter');
          Writeln;
          Writeln('         2. The Basic Filter with Smoothing (for Trend-Cycle Estimation)');
          Writeln;
          Writeln('         3. The Double Offset Filter');
          Writeln;
          Writeln('         4. The Triple Filter with Offsets');
          Writeln;
          Writeln('         5.  RETURN to the Main Menu');
          Writeln;
          Writeln('                      Select a number and press RETURN');

          GetBoundedInteger(1, 5, theMenuItem);

        until theMenuItem in [1..5];

{We must replace dataBOX dataVector by spectraBox spectraVector}

        case theMenuItem of
          1: {in RESPONSE}
            OrdinaryFilterSpec(filterSpec, rhoVec, sigmaVec, phi, theta, spectraVec, spectraBOX, SCREENBOX);
          2:  {in RESPONSE}
            SmoothingFilterSpec(filterSpec, rhoVec, sigmaVec, phi, theta, theta2, spectraVec, spectraBOX, SCREENBOX);
          3: {in RESPONSE}
            TwoPassFilterSpec(filterSpec, rhoVec, sigmaVec1, sigmaVec2, phi1, theta1, phi2, theta2, spectraVec, spectraBOX, SCREENBOX);
          4:  {in RESPONSE}
            TripleFilterSpec(filterSpec, sigmaVec, sigmaVec1, sigmaVec2, phi, theta, phi1, theta1, phi2, theta2, spectraVec, spectraBOX, SCREENBOX);
        end;{case}

      end; {reSpecify}

  end; {SpecifyTheFilter}

  {________________________________________________}

    procedure DetrendTheData (var dataBOX, residueBOX: rBOX;
                    var SCREENBOX, POSTBOX: IBOX;
                    var dataVector, residueVec: longVector);
      var
        theMenuItem: integer;

    begin

      if noPageParameters then
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}

      if (thereIsData = false) then
        begin
         SysBeep(3);
         Writeln('    First you must read some data!');
         Readln;
         GetTheData (dataBOX, SCREENBOX, POSTBOX, dataVector);
        end
      else
       repeat
        ClrScr;

        Writeln;
        Writeln('    DETREND THE DATA');
        Writeln;
        Writeln('         1. Extract a Polynomial Trend');
        Writeln;
        Writeln('         2. Specify the Butterworth Filter');
        Writeln;
        Writeln('         3. Apply the Butterworth Filter');
        Writeln;
        Writeln('         4. Return to the Main Menu');
        Writeln;
        Writeln('                      Select a number and press RETURN');

        GetBoundedInteger(1, 4, theMenuItem);

        case theMenuItem of
          1: {in DATASEG }
            FitPolyTrend(polyParas, dataBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, residueVec);
          2:  {in RESPONSE}
            ButterworthSpec(filterSpec, spectraBOX, spectraVec, SCREENBOX, POSTBOX);
          3: {in ORGANISE}
            if noButterworthSpec then
              begin
                Writeln('    First, you must specify the filter');
                Readln;
              end
            else
              ApplyButterFilter(dataBOX, trendBOX, residueBOX, dataVector, trendVector, residueVec, filterSpec);
        end;{case}

      until theMenuItem = 4;

    end; {DetrendTheData}

{_________________________________________________}

    procedure SaveComponents (var dataBOX, residueBOX, seasonBOX: rBOX;
                    var dataVector, residueVec, seasonVec, adjustedVec: longVector);

    var
        theMenuItem: integer;
        exitChar: char;
        exitNo: integer;
        doAntiLogs: boolean;

    begin


      exitChar := '2';
      exitNo := 2;

      repeat

        if logsTaken then
          begin {if logsTaken}
            ClrScr;

            Writeln;
            Writeln('    SAVE THE DATA COMPONENTS');
            Writeln;
            Writeln;
            Writeln('    The data sequence has  been  subjected to a logarithmic transformation.');
            Writeln('    You  many wish to apply anti-logarithms to the data and the components.');
            Writeln;
            Writeln('                                                                   <RETURN>');
            Readln;
          end;

        ClrScr;

        Writeln;
        Writeln('    SAVE THE DATA COMPONENTS');

        if thereIsData = false then
          begin
            Writeln;
            Writeln('        There are no data');
            theMenuItem := 1;
            exitNo := 1;
            exitChar := '1';
            Readln;
          end;


        Writeln;
        Writeln;
        if thereIsData then
          Writeln('         1. Save the Data Sequence');
        Writeln;

        if weHavePolyResiduals then
          begin
            Writeln('         2. Save the Polynomial Residuals');
            Writeln;
            exitChar := '3';
            exitNo := 3;
            if weHaveSeasonalComponent then
              begin
                Writeln('         3. Save the Seasonal Component');
                Writeln;
                Writeln('         4. Save the Seasonally Adjusted Data');
                Writeln;
                exitChar := '5';
                exitNo := 5;
              end;
          end;   {weHavePolyResiduals}

        if weHaveButterworthResiduals then
          begin
            Writeln('         2. Save the Trend-Cycle Function');
            Writeln;
            exitChar := '3';
            exitNo := 3;
            if weHaveSeasonalComponent then
              begin
                Writeln('         3. Save the Seasonal Component');
                Writeln;
                Writeln('         4. Save the Residue of the Threefold Decomposition');
                Writeln;
                exitChar := '5';
                exitNo := 5;
              end;
          end;  {weHaveButterworthResiduals}

        if (exitNo > 1) then
          Writeln('         ', exitChar, '. Return to the Main Menu');

        Writeln;
        Writeln('                      Select a number and press RETURN');

        if (exitNo <> 1) then
          GetBoundedInteger(1, exitNo, theMenuItem);

        if (theMenuItem = 1) and (exitNo <> 1) then
          begin
            YesOrNo('    Do you wish to take anti-logarithms?', doAntiLogs);
            if doAntiLogs then
              AntiLogOfVector(dataBOX, dataVector);
            WriteData(dataBOX, dataVector);
          end;

        if (theMenuItem = 3) and (exitNo <> 3) then
          begin
            YesOrNo('    Do you wish to take anti-logarithms?', doAntiLogs);
            if doAntiLogs then
               AntiLogOfVector(seasonBOX, seasonVec);
            WriteData(seasonBOX, seasonVec);
          end;

        if weHavePolyResiduals then
          begin
            if theMenuItem = 2 then
              begin
               YesOrNo('    Do you wish to take anti-logarithms?', doAntiLogs);
                if doAntiLogs then
                   AntiLogOfVector(residueBOX, residueVec);
                WriteData(residueBOX, residueVec);
              end;

            if theMenuItem = 4 then
              begin
                YesOrNo('    Do you wish to take anti-logarithms?', doAntiLogs);
                if doAntiLogs then
                   AntiLogOfVector(seasonBOX, adjustedVec);
                WriteData(dataBOX, adjustedVec);
                   {AntiLogOfVector(seasonBOX, seasonVec);
                WriteData(seasonBOX, seasonVec);}
               end;
          end; {weHavePolyResiduals}

        if weHaveButterworthResiduals then
          begin
            if theMenuItem = 2 then
              begin
                YesOrNo('    Do you wish to take anti-logarithms?', doAntiLogs);
                if doAntiLogs then
                   AntiLogOfVector(dataBOX, trendVector);
                WriteData(dataBOX, trendVector);
              end;

            if theMenuItem = 4 then
              begin
                YesOrNo('    Do you wish to take anti-logarithms?', doAntiLogs);
                if doAntiLogs then
                   AntiLogOfVector(residueBOX, adjustedVec);
                WriteData(residueBOX,adjustedVec);
              end;
          end;  {weHaveButterworthResiduals}

      until theMenuItem = exitNo;

    end; {SaveTheComponents}

{________________________________________________}

    procedure SaveAndExit (var dataBOX, residueBOX, seasonBOX: rBOX;
                    var dataVector, residueVec, seasonVec, adjustedVec: longVector);

      var
        theMenuItem: integer;

    begin
      repeat
        ClrScr;

        Writeln;
        Writeln('    SAVE AND EXIT');
        Writeln;
        Writeln('         1. Save the Data Components');
        Writeln;
        Writeln('         2. Return to the Main Menu');
        Writeln;
        Writeln('         3. EXIT the Program');
        Writeln;
        Writeln('                      Select a number and press RETURN');

        GetBoundedInteger(1, 3, theMenuItem);
      until theMenuItem in [1..3];

      case theMenuItem of
        1:
          SaveComponents(dataBOX, residueBOX, seasonBOX, dataVector, residueVec, seasonVec, adjustedVec);
        2:
          allsDone := false; {Return to the Main Menu'}
        3:
          allsDone := true;
      end;{cases}

    end;{SaveAndExit}

{________________________________________________}

  procedure TransformTheData (var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, resideVec: longVector;
                  var polyParas: polyParameters);


  {THIS PROCEDURE IS NOT CURRENTlY ACTIVE}

    var
      theMenuItem: integer;

  begin
    if thereisData = false then
      GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector)
    else
    repeat
      ClrScr;

      Writeln;
      Writeln('    TRANSFORM THE DATA');
      Writeln;
      Writeln('         1. Take Logarithms of the Data');
      Writeln;
      Writeln('         2. Extract a Polynomial Trend');
      Writeln;
      Writeln('         3. Plot the Data');
      Writeln;
      Writeln('         4. Return to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press RETURN');


      GetBoundedInteger(1, 4, theMenuItem);

      if thereisData = false then
        ReadTheData(dataBOX, dataVector)
      else
      case theMenuItem of
        1:
          if thereisData then
          begin
           if (logsTaken = false) then
             begin
               LogOfVector(dataBOX, dataVector); {in DATASEG}
               Writeln('         The data have been replaced by their logarithms');
               Readln;
             end
           else
             begin
               Writeln('         You have already taken logarithms of the data');
               Readln;
             end;
          end;
        2:
          FitPolyTrend(polyParas, dataBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, residueVec);
        3: 
          PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in DATASEG}
      end;{case}

    until theMenuItem = 4;

  end; {TransformTheData }

{________________________________________________}


procedure DoSeasonalAdjustment (var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, resideVec: longVector;
                  var polyParas: polyParameters);

    var
      theMenuItem: integer;

  begin
    repeat
      ClrScr;

      Writeln;
      Writeln('    DO SEASONAL ADJUSTMENT');
      Writeln;
      Writeln('         1. Specify the Seasonal-Adjustment Filter');
      Writeln;
      Writeln('         2. Apply the Seasonal-Adjustment Filter');
      Writeln;
      Writeln('         3. Return to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press RETURN');

      GetBoundedInteger(1, 3, theMenuItem);

      case theMenuItem of
        1: 
          SpecifyTheFilter(filterSpec, spectraVec, spectraBOX, rhoVec, phi, phi1, phi2, theta, theta1, theta2);
        2:
          if noSeaFilterSpec then
            begin
              Writeln('    First, you must specify the filter');
              Readln;
            end
          else
            ApplyTheFilters(filterSpec, phi, phi1, phi2, theta, theta1, theta2, spectraBOX, dataBOX, residueBOX, spectraVec, dataVector, residueVec, seasonVec);
      end;{case}

    until theMenuItem = 3;
  end; {DoSeasonalAdjustment}

{________________________________________________}

  procedure ApplyTheFilters (var filterSpec: filterRecord;
                  var phi, phi1, phi2, theta, theta1, theta2: seasVector;
                  var spectraBOX, dataBOX, residueBOX: rBOX;
                  var spectraVec, dataVector, residueVec, seasonVec: longVector);

{called by <DoSeasonalAdjustment> in ORGANISE}
    var
      s, Tcap: integer;
      legend: string;

  begin
    s := filterSpec.s;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}

    if thereIsData = false then
      GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}

    if dataAreTrended and (not weHaveTrendResiduals) then
      FitPolyTrend(polyParas, dataBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, residueVec);

    Tcap := residueBOX.NOPUNCT;
    seasonVec := residueVec; {Vector of seasonal fluctuations}
    adjustedVec := residueVec; {Seasonally adjusted vector}

    if noSeaFilterSpec then {in ORGANISE}
      begin
        SpecifyTheFilter(filterSpec, spectraVec, spectraBOX, rhoVec, phi, phi1, phi2, theta, theta1, theta2);
        s := filterSpec.seasonality;
      end;

    if  filterSpec.filterType = singleFilter then  {in FILTERS}
      ApplySingleSeasonalFilter(s, Tcap, phi, sigmaVec, seasonVec, adjustedVec);

    if  filterSpec.filterType = TaperedFilter then  {in FILTERS}
      ApplySmoothingFilter(s, q, Tcap, phi, phi2, sigmaVec, muVec, seasonVec, adjustedVec);

    if  filterSpec.filterType = doubleFilter then  {in FILTERS}
      ApplyDoubleSeasonalFilter(s, Tcap, phi1, phi2, sigmaVec1, sigmaVec2, seasonVec, adjustedVec);

    if  filterSpec.filterType = tripleFilter then  {in FILTERS}
      ApplyTripleSeasonalFilter(s, Tcap, phi, phi1, phi2, sigmaVec, sigmaVec1, sigmaVec2, seasonVec, adjustedVec);

    MakeDataBox(residueBOX, residueVec);

    if weHaveAdjustedResiduals then
      legend := DeSeasonalisedOverlay(dataBOX, dataName);

    if weHaveCycleComponent then
      legend := SmoothCycleOverlay(dataBOX, dataName);

    if  weHaveButterworthResiduals then
      legend := ButterworthOverlay;

    ScreenOverplot(residueBOX, SCREENBOX, residueVec, adjustedVec, legend);

    if (PSPict = true) then
      WritePostOverFile(residueBOX, POSTBOX, residueVec, adjustedVec, legend);

    if weHaveButterworthResiduals then
      begin
        MakeDataBox(residueBOX, adjustedVec);
        legend := 'The residual vector of a threefold decomposition the data, plotted separately';
         ScreenMakePlot(residueBOX, SCREENBOX, adjustedVec, legend);
         if (PSPict = true) then
           WritePostFile(residueBOX, POSTBOX, adjustedVec, legend);
      end;

    seasonBOX := residueBOX;
    MakeDataBox(seasonBOX, seasonVec);

    if weHaveSeasonalComponent then
      legend := SeasonalComponent(dataBOX, dataName);  {takes account of thr threefold decomposition}

    if weHaveCycleComponent then
        legend := TrendCycleResidue(dataBOX, dataName);


    ScreenMakePlot(seasonBOX, SCREENBOX, seasonVec, legend);

    if (PSPict = true) then
           WritePostFile(seasonBOX, POSTBOX, seasonVec, legend);


    if wehavePolyResiduals then   {in FILTERS}
      TrendPlusSeason(polyParas, dataBOX, residueBOX, dataVector, adjustedVec, SCREENBOX, POSTBOX);

    weHaveSeasonalComponent := true;

  end;{ApplyTheFilters}

  {_________________________________________________}

    procedure ApplyButterFilter (var dataBOX, trendBOX, residueBOX: rBOX;
                    var dataVector, trendVector, residueVec: longVector;
                    var filterSpec: FilterRecord);

      var
        legend: string;

    begin
      if thereIsData = false then
        begin
          SysBeep(3);
          GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector);
          {ReadTheData(dataBOX, dataVector);}
        end;

      if noPageParameters then
        begin
          SysBeep(3);
          PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
        end;

      if noButterworthSpec then
        ButterworthSpec(filterSpec, spectraBOX, spectraVec, SCREENBOX, POSTBOX);

      MakeButterTrend(dataBOX, trendBOX, residueBOX, filterSpec, dataVector, trendVector, residueVec);

      {residueBOX := dataBOX : Here we should plot the Butterworth cycle}
      MakeDataBox (residueBOX, residueVec);
      {ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);}
      legend := ButterworthTrend(dataBOX, dataName, filterSpec);
      ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);

      if (PSPict = true) then
        WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);

      legend := ButterworthResiduals(dataBOX, dataName, filterSpec);
      ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

      if (PSPict = true) then
        WritePostFile(residueBOX, POSTBOX, residueVec, legend);

  {weHaveEstimatedATrend := true;}
      weHaveTrendResiduals := true;
      wehavePolyResiduals := false;
      weHaveButterworthResiduals := true;
      noSeaFilterSpec := true;


    end; {ApplyButterFilter}

  {________________________________________________}

end. {ORGANISE: Unit 11 of SEASIDE.PAS}

