unit RESPONSE; {Unit 8 of SEASIDE.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, LEGENDS, FILTER, SCREEN, POST, DATASEG, Dos, Crt, Graph;

  procedure MakeSigmaVec (var sigmaVec: seasVector;
                  var s: integer);

  procedure MakeRhoVec (var rhoVec: seasVector;
                  var rho: real;
                  var s: integer);

  procedure QuarterlyQuads (var sigmaVec, rhoVec: seasVector;
                  var rho, zeta: real;
                  var quadNo: integer);

  procedure CompoundQuads (var sigmaVec, rhoVec: seasVector;
                  var rho, zeta: real;
                  var quadNo: integer);

  procedure NewQuarterlyQuads (var sigmaVec, rhoVec: seasVector;
                  var zeta: vector;
                  var rho: real;
                  var quadNo: integer);

  procedure NewCompoundQuads (var sigmaVec, rhoVec: seasVector;
                  var zeta: vector;
                  var rho: real;
                  var quadNo: integer);

  procedure WKCoefficients (var phi, theta: seasVector;
                  q: integer;
                  var lambda: real);

  procedure GetSmoothingParameters (var muVec, theta2: seasVector;
                  var q: integer);

  function MuResponse (var theta2: seasVector;
                  var q: integer;
                  var omega: real): real;

  function TrendCycleResponse (var phi, theta, theta2: seasVector;
                  var s, q: integer;
                  var omega: real): real;

  function SeasResponse (var phi, theta: seasVector;
                  var s: integer;
                  var omega: real): real;

  function DualSeaResponse (var phi1, phi2, theta1, theta2: seasVector;
                  var s: integer;
                  var omega: real): real;

  function TripleSeaResponse (var phi0, phi1, phi2, theta0, theta1, theta2: seasVector;
                  var s: integer;
                  var omega: real): real;

  function ButterworthResponse (var filterSpec: FilterRecord;
                  var omega: real): real;

  procedure GetFilterGainVector(var filterSpec: FilterRecord;
                  var SCREENBOX: IBOX;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var phi, theta, phi1, theta1, phi2, theta2: seasVector;
                  var s, q: integer);

{++++++++++++++++++++++++++++++++++++++++}
  procedure CentralFilterVec (var phi, theta: seasVector;
                  var sigmaVec, rhoVec: seasVector;
                  var lambda, rho: real;
                  s: integer);

  procedure OrdinaryFilterSpec (var filterSpec: FilterRecord;
                  var rhoVec, sigmaVec, phi, theta: seasVector;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure SmoothingFilterSpec (var filterSpec: FilterRecord;
                  var rhoVec, sigmaVec, phi, theta, theta2: seasVector;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure TwoPassFilterSpec (var filterSpec: FilterRecord;
                  var rhoVec, sigmaVec1, sigmaVec2: seasVector;
                  var phi1, theta1, phi2, theta2: seasVector;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure GetTheOffsets (var filterSpec: FilterRecord;
                  var zetaSub, zetaSup: vector;
                  var quadNo: integer);

  procedure OffsetFilterVecs (var filterSpec: FilterRecord;
                  var sigmaVec1, sigmaVec2: seasVector;
                  var phi1, theta1, phi2, theta2: seasVector;
                  var lambda, rho: real;
                  var quadNo: integer);

  procedure TripleFilterSpec (var filterSpec: FilterRecord;
                  var sigmaVec, sigmaVec1, sigmaVec2: seasVector;
                  var phi, theta, phi1, theta1, phi2, theta2: seasVector;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure ButterworthSpec (var filterSpec: FilterRecord;
                  var spectraBOX: rBOX;
                  var spectraVec: longVector;
                  var SCREENBOX, POSTBOX: IBOX);

implementation

 {_________________________________________________}

  procedure MakeSigmaVec (var sigmaVec: seasVector;
                  var s: integer);

{This produces the coefficients of the unit root polynomial Sigma(z) }

    var
      i: integer;
  begin
    for i := 0 to s - 1 do
      sigmaVec[i] := 1.0;
  end; {MakeSigmaVec}

{_________________________________________________}

  procedure MakeRhoVec (var rhoVec: seasVector;
                  var rho: real;

{This produces the coefficients of the denominator  polynonial Rho(z),}
{which has to be compouned to produce  the W-K denominator  }
{phi ( z ) = Sigma(z)Sigma(z^[-1]) + lambda*Rho(z)Rho(z^[-1])  }
                  var s: integer);
    var
      i: integer;
      rhoPower: real;

  begin
    rhoPower := 1.0;
    for i := 0 to s - 1 do
      begin
        rhoVec[i] := rhoPower;
        rhoPower := rhoPower * rho;
      end;
  end; {MakeRhoVec}

{_________________________________________________}

  procedure QuarterlyQuads (var sigmaVec, rhoVec: seasVector;
                  var rho, zeta: real;
                  var quadNo: integer);

{This procedure is for generating the coefficents of polynomials Sigma(z)}
{and Rho(z) for a quarterly filter with the same unique offset from each}
{of the seasonal frequencies. It is used in the Dual Filter}

    var
      quadZeros, quadPoles: seasMatrix;
      omega: seasVector;
      degree: extended;
      alpha: seasVector;
      j, p, k: integer;
      zetaR: real;

  begin

    degree := pi / 180;
    omega[1] := pi / 2.0;
    omega[2] := pi;

    zetaR := zeta * degree;

    QuadZeros[1, 0] := 1.0;
    QuadZeros[1, 1] := -2.0 * Cos(omega[1] + zetaR);
    QuadZeros[1, 2] := 1.0;

    QuadZeros[2, 0] := 1.0;
    if  filterSpec.filterType = doubleFilter then
      begin
        QuadZeros[2, 1] := -2.0 * Cos(pi + zetaR);
        QuadZeros[2, 2] := 1.0;
      end;
    if  filterSpec.filterType = singleFilter then
      begin
        QuadZeros[1, 1] := 1.0;
        QuadZeros[1, 2] := 0.0;
      end;

    sigmaVec[0] := QuadZeros[1, 0];
    sigmaVec[1] := QuadZeros[1, 1];
    sigmaVec[2] := QuadZeros[1, 2];

    p := 2;
    k := 2; {or is it unity?}
    for j := 2 to quadNo do
      begin
        alpha[0] := quadZeros[j, 0];
        alpha[1] := quadZeros[j, 1];
        alpha[2] := quadZeros[j, 2];
        Convolution(alpha, sigmaVec, p, k);
        k := k + 2
      end;

{++++++++++++++++++++++++++++++++++++++}
    QuadPoles[1, 0] := 1.0;
    QuadPoles[1, 1] := -2.0 * rho * Cos(omega[1] + zetaR);
    QuadPoles[1, 2] := rho * rho;

    QuadPoles[2, 0] := 1.0;
    if  filterSpec.filterType = DoubleFilter then
      begin
        QuadPoles[2, 1] := -2.0 * rho * Cos(pi + zetaR);
        QuadPoles[2, 2] := rho * rho;
      end;
    if  filterSpec.filterType = singleFilter then
      begin
        QuadPoles[2, 1] := rho;
        QuadPoles[2, 2] := 0.0;
      end;

    rhoVec[0] := quadPoles[1, 0];
    rhoVec[1] := quadPoles[1, 1];
    rhoVec[2] := quadPoles[1, 2];

    p := 2;
    k := 2; {or is it unity?}
    for j := 2 to quadNo do
      begin
        alpha[0] := QuadPoles[j, 0];
        alpha[1] := QuadPoles[j, 1];
        alpha[2] := QuadPoles[j, 2];
        Convolution(alpha, rhoVec, p, k);
        k := k + 2
      end;

  end; {QuarterlyQuads}

{_________________________________________________}

  procedure CompoundQuads (var sigmaVec, rhoVec: seasVector;
                  var rho, zeta: real;
                  var quadNo: integer);

{This procedure is for generating the coefficents of polynomials Sigma(z)}
{and Rho(z) for a monthly filter with the same unique offset from each}
{of the seasonal frequencies. It is used in the Dual Filter}

    var
      quadZeros, quadPoles: seasMatrix;
      omega: seasVector;
      degree: extended;
      alpha: seasVector;
      j, p, k: integer;
      zetaR: real;

  begin

    degree := pi / 180;
    omega[1] := pi / 6.0;
    omega[2] := pi / 3.0;
    omega[3] := pi / 2.0;
    omega[4] := 2.0 * pi / 3.0;
    omega[5] := 5.0 * pi / 6.0;

    zetaR := zeta * degree;

{These relate to the zero frequency and they are not active}
    QuadZeros[0, 0] := 1.0;
    QuadZeros[0, 1] := -1.0;
    QuadZeros[0, 2] := 0.0;

    for j := 1 to 5 do
      begin
        QuadZeros[j, 0] := 1.0;
        QuadZeros[j, 1] := -2.0 * Cos(omega[j] + zetaR);
        QuadZeros[j, 2] := 1.0;
      end;

    QuadZeros[6, 0] := 1.0;
    if  filterSpec.filterType = DoubleFilter then
      begin
        QuadZeros[6, 1] := -2.0 * Cos(pi + zetaR);
        QuadZeros[6, 2] := 1.0;
      end;
    if  filterSpec.filterType = singleFilter then
      begin
        QuadZeros[6, 1] := 1.0;
        QuadZeros[6, 2] := 0.0;
      end;

    sigmaVec[0] := QuadZeros[1, 0];
    sigmaVec[1] := QuadZeros[1, 1];
    sigmaVec[2] := QuadZeros[1, 2];

    p := 2;
    k := 2; {or is it unity?}
    for j := 2 to quadNo do
      begin
        alpha[0] := quadZeros[j, 0];
        alpha[1] := quadZeros[j, 1];
        alpha[2] := quadZeros[j, 2];
        Convolution(alpha, sigmaVec, p, k);
        k := k + 2
      end;

{++++++++++++++++++++++++++++++++++++++}

{These are not active}
    quadPoles[0, 0] := 1.0;
    quadPoles[0, 1] := -rho;
    quadPoles[0, 2] := 0.0;

    for j := 1 to 5 do
      begin
        quadPoles[j, 0] := 1.0;
        quadPoles[j, 1] := -2.0 * rho * Cos(omega[j] + zetaR);
        quadPoles[j, 2] := rho * rho;
      end;

    quadPoles[6, 0] := 1.0;
    if  filterSpec.filterType = DoubleFilter then
      begin
        quadPoles[6, 1] := -2.0 * rho * Cos(pi + zetaR);
        quadPoles[6, 2] := rho * rho;
      end;
    if  filterSpec.filterType = singleFilter then
      begin
        quadPoles[6, 1] := rho;
        quadPoles[6, 2] := 0.0;
      end;

    rhoVec[0] := quadPoles[1, 0];
    rhoVec[1] := quadPoles[1, 1];
    rhoVec[2] := quadPoles[1, 2];

    p := 2;
    k := 2;
    for j := 2 to quadNo do
      begin
        alpha[0] := quadPoles[j, 0];
        alpha[1] := quadPoles[j, 1];
        alpha[2] := quadPoles[j, 2];
        Convolution(alpha, rhoVec, p, k);
        k := k + 2
      end;

    noModel := false;

  end; {CompoundQuads}
{_________________________________________________}

  procedure NewQuarterlyQuads (var sigmaVec, rhoVec: seasVector;
                  var zeta: vector;
                  var rho: real;
                  var quadNo: integer);

{This procedure is for generating the coefficents of the polynomials}
{Sigma(z)and Rho(z) for a quarterly filter with different offsets from}
{ each of the seasonal frequencies. It is used  the Triple Filter}

    var
      quadZeros, quadPoles: seasMatrix;
      omega: seasVector;
      degree: extended;
      zetaR: reaL;
      alpha: seasVector;
      j, p, k: integer;

  begin

    degree := pi / 180;
    omega[1] := pi / 2.0;
    omega[2] := pi;

    zetaR := zeta[1] * degree;
    QuadZeros[1, 0] := 1.0;
    QuadZeros[1, 1] := -2.0 * Cos(omega[1] + zetaR);
    QuadZeros[1, 2] := 1.0;

    QuadZeros[2, 0] := 1.0;
    if offset = true then
      begin
        zetaR := zeta[2] * degree;
        QuadZeros[2, 1] := -2.0 * Cos(pi + zetaR);
        QuadZeros[2, 2] := 1.0;
      end
    else
      begin
        QuadZeros[2, 1] := 1.0;
        QuadZeros[2, 2] := 0.0;
      end;

    sigmaVec[0] := QuadZeros[1, 0];
    sigmaVec[1] := QuadZeros[1, 1];
    sigmaVec[2] := QuadZeros[1, 2];

    p := 2;
    k := 2;
    for j := 2 to quadNo do
      begin
        alpha[0] := quadZeros[j, 0];
        alpha[1] := quadZeros[j, 1];
        alpha[2] := quadZeros[j, 2];
        Convolution(alpha, sigmaVec, p, k);
        k := k + 2
      end;

{++++++++++++++++++++++++++++++++++++++}

    zetaR := zeta[1] * degree;
    quadPoles[1, 0] := 1.0;
    quadPoles[1, 1] := -2.0 * rho * Cos(omega[1] + zetaR);
    quadPoles[1, 2] := rho * rho;


    quadPoles[2, 0] := 1.0;
    if offset then
      begin
        zetaR := zeta[2] * degree;
        quadPoles[2, 1] := -2.0 * rho * Cos(omega[2] + zetaR);
        quadPoles[2, 2] := rho * rho;
      end
    else
      begin
        quadPoles[2, 1] := rho;
        quadPoles[2, 2] := 0.0;
      end;

    rhoVec[0] := quadPoles[1, 0];
    rhoVec[1] := quadPoles[1, 1];
    rhoVec[2] := quadPoles[1, 2];

    p := 2;
    k := 2;
    for j := 2 to quadNo do
      begin
        alpha[0] := quadPoles[j, 0];
        alpha[1] := quadPoles[j, 1];
        alpha[2] := quadPoles[j, 2];
        Convolution(alpha, rhoVec, p, k);
        k := k + 2
      end;

    noModel := false;


  end; {NewQuarterlyQuads}

{_________________________________________________}

  procedure NewCompoundQuads (var sigmaVec, rhoVec: seasVector;
                  var zeta: vector;
                  var rho: real;
                  var quadNo: integer);

{This procedure is for generating the coefficents of polynomials}
{Sigma(z)and Rho(z) for a monthly filter with different offsets from}
{each of the seasonal frequencies. It is used in the Triple Filter}

    var
      quadZeros, quadPoles: SeasMatrix;
      omega: seasVector;
      degree: extended;
      zetaR: reaL;
      alpha: seasVector;
      j, p, k: integer;

  begin

    degree := pi / 180;
    omega[1] := pi / 6.0;
    omega[2] := pi / 3.0;
    omega[3] := pi / 2.0;
    omega[4] := 2.0 * pi / 3.0;
    omega[5] := 5.0 * pi / 6.0;

    QuadZeros[0, 0] := 1.0;
    QuadZeros[0, 1] := -1.0;
    QuadZeros[0, 2] := 0.0;

    for j := 1 to 5 do
      begin
        zetaR := zeta[j] * degree;
        QuadZeros[j, 0] := 1.0;
        QuadZeros[j, 1] := -2.0 * Cos(omega[j] + zetaR);
        QuadZeros[j, 2] := 1.0;
      end;

    QuadZeros[6, 0] := 1.0;
    if offset = true then
      begin
        zetaR := zeta[6] * degree;
        QuadZeros[6, 1] := -2.0 * Cos(pi + zetaR);
        QuadZeros[6, 2] := 1.0;
      end
    else
      begin
        QuadZeros[6, 1] := 1.0;
        QuadZeros[6, 2] := 0.0;
      end;

    sigmaVec[0] := QuadZeros[1, 0];
    sigmaVec[1] := QuadZeros[1, 1];
    sigmaVec[2] := QuadZeros[1, 2];

    p := 2;
    k := 2;
    for j := 2 to quadNo do
      begin
        alpha[0] := quadZeros[j, 0];
        alpha[1] := quadZeros[j, 1];
        alpha[2] := quadZeros[j, 2];
        Convolution(alpha, sigmaVec, p, k);
        k := k + 2
      end;

{++++++++++++++++++++++++++++++++++++++}

    quadPoles[0, 0] := 1.0;
    quadPoles[0, 1] := -rho;
    quadPoles[0, 2] := 0.0;

    for j := 1 to 5 do
      begin
        zetaR := zeta[j] * degree;
        quadPoles[j, 0] := 1.0;
        quadPoles[j, 1] := -2.0 * rho * Cos(omega[j] + zetaR);
        quadPoles[j, 2] := rho * rho;
      end;

    quadPoles[6, 0] := 1.0;
    if offset then
      begin
        zetaR := zeta[6] * degree;
        quadPoles[6, 1] := -2.0 * rho * Cos(pi + zetaR);
        quadPoles[6, 2] := rho * rho;
      end
    else
      begin
        quadPoles[6, 1] := rho;
        quadPoles[6, 2] := 0.0;
      end;

    rhoVec[0] := quadPoles[1, 0];
    rhoVec[1] := quadPoles[1, 1];
    rhoVec[2] := quadPoles[1, 2];

    p := 2;
    k := 2;
    for j := 2 to quadNo do
      begin
        alpha[0] := quadPoles[j, 0];
        alpha[1] := quadPoles[j, 1];
        alpha[2] := quadPoles[j, 2];
        Convolution(alpha, rhoVec, p, k);
        k := k + 2
      end;

    noModel := false;

  end; {NewCompoundQuads}

{_________________________________________________}

  procedure WKCoefficients (var phi, theta: seasVector;
                  q: integer;
                  var lambda: real);

{This procedure forms the Wiener-Kolmogorov coefficients from the parameters}
{phi and theta, and it normalises the gain of the W-K filter at zero frequency.}
{The procedure is used in <CentralFilterVec>, <TwoPassFilterSpec> and <OffsetFilterVecs>.}
{phi becomes the denominator of the W-K filter}

    var
      j: integer;
      thetaSum, phiSum, factor: extended;

  begin

{Make lambda* R(z)R(1/z) + S(z)S(1/z)}
    for j := 0 to q do
      phi[j] := lambda * phi[j] + theta[j];

    thetaSum := theta[0];
    phiSum := phi[0];
    for j := 1 to q do
      begin
        thetaSum := thetaSum + 2 * theta[j];
        phiSum := phiSum + 2 * phi[j];
      end;

    factor := thetaSum / phiSum;
    for j := 0 to q do
      phi[j] := phi[j] * factor;

  end; {WKCoefficients}

{_________________________________________________}

  function SeasResponse (var phi, theta: seasVector;
                  var s: integer;
                  var omega: real): real;
    var
      numerator, denominator: extended;
      j: integer;

  begin
    numerator := theta[0];
    denominator := phi[0];
    for j := 1 to s - 1 do
      begin
        numerator := numerator + 2 * theta[j] * cos(j * omega);
        denominator := denominator + 2 * phi[j] * cos(j * omega);
      end;
    SeasResponse := numerator / denominator;

  end; {SeasResponse}

{_________________________________________________}

  procedure GetSmoothingParameters (var muVec, theta2: seasVector;
                  var q: integer);

    var
      muNo, thetaSum, muSum: real;
      filterChoice: char;
      i: integer;

  begin
    ClrScr;

    Writeln;
    Writeln;
    Writeln('    THE SMOOTHING FILTER (FOR THE TREND-CYCLE FUNCTION)');
    Writeln;
    Writeln('    The smoothing  operation  uses the bi-directional filters M(z^-1)N(z)');
    Writeln('    or M^2(z^-1)M^2(z) that are  based on the second-order moving average');
    Writeln('    M(z) = (1+ z)(1 + mu z)  = 1 + (1 + mu) z + muz^2, which incorporates');
    Writeln('    a zero at the limiting  Nyquist  frequency of pi and  another zero in');
    Writeln('    the interval; [pi/2, pi].');
    Writeln;

    repeat
      Writeln('    Choose  ');
      Writeln;
      Writeln('       (A) The bi-directional filter M(z^-1)N(z) or ');
      Writeln;
      Writeln('       (B) its "squared" version M^2(z^-1)M^2(z)');
      Readln(filterChoice);
    until filterChoice in ['a', 'A', 'b', 'B'];

    if filterChoice in ['a', 'A'] then
      q := 2;
    if filterChoice in ['b', 'B'] then
      q := 4;

    Writeln('    Specify the value of mu (0.7 works well)');
    ReadBoundedNumber(0.0, 1.0, muNo);

    if q = 2 then
      begin
        muVec[0] := 1.0;
        muVec[1] := 1.0 + muNo;
        muVec[2] := muNo;
      end;

    if q = 4 then
      begin
        muVec[0] := 1.0;
        muVec[1] := 2 * (1.0 + muNo);
        muVec[2] := 2 * Sqr(muNo) + Sqr(1.0 + muNo);
        muVec[3] := 2 * Sqr(muNo) * (1.0 + muNo);
        muVec[4] := Sqr(muNo) * Sqr(muNo);
      end;

{We can compound these parameters using Laurent}
    Laurent(muVec, theta2, q);

{Then we must ensure that the filters have unit gain at zero frequency}
    thetaSum := theta2[0];
    muSum := 1.0;
    for i := 1 to q do
      begin
        muSum := muSum + muVec[i];
        thetaSum := thetaSum + 2 * theta2[i];
      end;

    for i := 0 to q do
      begin
        muVec[i] := muVec[i] / muSum;
        theta2[i] := theta2[i] / thetaSum;
      end;

  end; {GetSmoothingParameters}

{_________________________________________________}

  function MuResponse (var theta2: seasVector;
                  var q: integer;
                  var omega: real): real;
    var
      numerator: extended;
      j: integer;

  begin

    numerator := theta2[0];
    for j := 1 to q do
      numerator := numerator + 2 * theta2[j] * Cos(j * omega);

    MuResponse := numerator;

  end; {MuResponse}

{_________________________________________________}

  function TrendCycleResponse (var phi, theta, theta2: seasVector;
                  var s, q: integer;
                  var omega: real): real;
    var
      numerator, denominator, firstResponse: extended;
      j: integer;

  begin
    numerator := theta[0];
    denominator := phi[0];
    for j := 1 to s - 1 do
      begin
        numerator := numerator + 2 * theta[j] * cos(j * omega);
        denominator := denominator + 2 * phi[j] * cos(j * omega);
      end;

    firstResponse := numerator / denominator;

    numerator := MuResponse(theta2, q, omega);

    TrendCycleResponse := firstResponse * numerator;

  end; {TrendCycleResponse }

{_________________________________________________}

  function DualSeaResponse (var phi1, phi2, theta1, theta2: seasVector;
                  var s: integer;
                  var omega: real): real;
    var
      numerator, denominator, firstResponse: extended;
      j: integer;

  begin

    firstResponse := 1.0;

    numerator := theta1[0];
    denominator := phi1[0];
    for j := 1 to s - 2 do {10 originally}
      begin
        numerator := numerator + 2 * theta1[j] * cos(j * omega);
        denominator := denominator + 2 * phi1[j] * cos(j * omega);
      end;
    firstResponse := numerator / denominator;

    numerator := theta2[0];
    denominator := phi2[0];
    for j := 1 to s do {12 originally}
      begin
        numerator := numerator + 2 * theta2[j] * cos(j * omega);
        denominator := denominator + 2 * phi2[j] * cos(j * omega);
      end;
    DualSeaResponse := firstResponse * numerator / denominator;

  end; {DualSeaResponse}

{_________________________________________________}

  function TripleSeaResponse (var phi0, phi1, phi2, theta0, theta1, theta2: seasVector;
                  var s: integer;
                  var omega: real): real;

{phi0, theta0 are for the central filter,}
{phi1, theta1,phi2, theta2. are for the offset filters}

    var
      numerator, denominator, firstResponse: extended;
      j: integer;

  begin

    firstResponse := 1.0;
    numerator := theta1[0];
    denominator := phi1[0];
    for j := 1 to s - 2 do {10 = s - 2 originally}
      begin
        numerator := numerator + 2 * theta1[j] * cos(j * omega);
        denominator := denominator + 2 * phi1[j] * cos(j * omega);
      end;
    firstResponse := numerator / denominator;

    numerator := theta2[0];
    denominator := phi2[0];
    for j := 1 to s do {s= 12 originally}
      begin
        numerator := numerator + 2 * theta2[j] * cos(j * omega);
        denominator := denominator + 2 * phi2[j] * cos(j * omega);
      end;
    firstResponse := firstResponse * numerator / denominator;

    numerator := theta0[0];
    denominator := phi0[0];
    for j := 1 to s - 1 do {11 =s-1 originally}
      begin
        numerator := numerator + 2 * theta0[j] * cos(j * omega);
        denominator := denominator + 2 * phi0[j] * cos(j * omega);
      end;
    TripleSeaResponse := firstResponse * numerator / denominator;


  end; {TripleSeaResponse}

  {_________________________________________________}

    function ButterworthResponse (var filterSpec: FilterRecord;
                    var omega: real): real;

  {ButterworthResponse  --> ButterworthResponse}

   {Gain or SquareGain?: It is the gain, Should this include FilterSpec?}
      var
        x, pi, gain: real;
        n, twoN: integer;

    begin
      pi := 4 * Arctan(1.0);
      with filterSpec do
        begin{filterSpec }
          n := filterOrder;
          x := Tan(omega / 2) * CoTan(CutOff / 2);

          twoN := 2 * n;
          gain := 1 / (1 + Power(x, twoN));
        end;{FilterRecord}
      ButterworthResponse := gain;

    end; {ButterworthResponse}

  {_________________________________________________}

  procedure GetFilterGainVector(var filterSpec: FilterRecord;
                  var SCREENBOX: IBOX;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var phi, theta, phi1, theta1, phi2, theta2: seasVector;
                  var s, q: integer);
  var
    BOXWDT, i: integer;
    yMax, yMin, omega, pi, inc: real;

  begin
    pi := 4.0 * Arctan(1.0);
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;

    inc := pi / BOXWDT;
    omega := 0.0;

    yMax := 0.0;
    yMin := 0.0;
    for i := 0 to BOXWDT do
      begin {i}
        if filterSpec.filterType = singleFilter then
          spectraVec[i] := SeasResponse(phi, theta, s, omega);
        if filterSpec.filterType = taperFunction then
          spectraVec[i] := MuResponse(theta2, q, omega);
        if filterSpec.filterType = taperedFilter then
          spectraVec[i] := TrendCycleResponse(phi, theta, theta2, s, q, omega);
        if filterSpec.filterType = DoubleFilter then
          spectraVec[i] := DualSeaResponse(phi1, phi2, theta1, theta2, s, omega);
        if filterSpec.filterType = tripleFilter then
          spectraVec[i] := TripleSeaResponse(phi, phi1, phi2, theta, theta1, theta2, s, omega);
        if filterSpec.filterType = Butterworth then
          spectraVec[i] := ButterworthResponse(filterSpec, omega);

        yMax := RMax(yMax, spectraVec[i]);
        yMin := RMin(yMin, spectraVec[i]);
        omega := omega + inc;

      end; {i}

    spectraBOX.NOPUNCT := BOXWDT;
{spectraBOX.xRange := BOXWDT;}
    spectraBOX.xMax := pi;
    spectraBOX.xMin := 0.0;
    spectraBOX.yMax := yMax;
    spectraBOX.yMin := yMin;
    spectraBOX.xRB := pi;
    spectraBOX.xLB := 0.0;
    spectraBOX.yUB := yMax;
    spectraBOX.yLB := yMin;

  end; {GetFilterGainVector}

{_________________________________________________}

  procedure CentralFilterVec (var phi, theta: seasVector;
                  var sigmaVec, rhoVec: seasVector;
                  var lambda, rho: real;
                  s: integer);

{This serves both for the ordinary filter and the ThreePass}
{filter. It effects the seasonal adjustment of the data}

  begin

    MakeSigmaVec(sigmaVec, s);
    MakeRhoVec(rhoVec, rho, s);
    Laurent(sigmaVec, theta, s - 1);
    Laurent(rhoVec, phi, s - 1);
    WKCoefficients(phi, theta, s - 1, lambda);

{Observe that Laurent and WKCoefficients are generic procedures}
{that are not specialised for seasonal adjustment. Therefore, they}
{take the parameter q = s - 1 instead of s}

  end; {CentralFilterVec}

{_________________________________________________}

  procedure OrdinaryFilterSpec (var filterSpec: FilterRecord;
                  var rhoVec, sigmaVec, phi, theta: seasVector;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX);

    var
      s, quadNo, sPlus: integer;
      pi: real;
      alpha, beta: fourierVec;
      plotBOX: rBOX;
      plotVector: longvector;
      F: text;
      legend: string;

  begin
    pi := 4.0 * Arctan(1.0);
     filterSpec.filterType := singleFilter;

{ARE THESE REDUNDANT?}
    offset := false;

    s := filterSpec.seasonality;
    quadNo := (s - 2) div 2;
    sPlus := s;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}

    ClrScr;

    Writeln;
    Writeln;
    Writeln('   THE ORDINARY SEASONAL-ADJUSTMENT FILTER FILTER');
    Writeln;
    Writeln('   The odinary seasonal adjustment filter comprises poles and zeros located');
    Writeln('   at precisely the seasonal frequencies, which are the fundamental seasonal');
    Writeln('   frequency and its harmonics  within  the interval [0, pi]. The  zeros lie');
    Writeln('   on the  circumference on the unit  circle and the poles, which lie on the');
    Writeln('   same radii, are at a  small distance  from the circle. The zeros serve to');
    Writeln('   nullify the elelements of the data  at the seasonal frequencies, and  the');
    Writeln('   poles are  to ensure that these effects are limited to  the vicinities of');
    Writeln('   the seasonal frequencies.');
    Writeln(' ' : 68, '<RETURN>');
    Readln;

    Writeln('    Specify value of the lambda (0.5 works well)');
    ReadBoundedNumber(0.0, 1.0, lambda);
    Writeln('    Specify value of the  rho (0.9 works well)');
    ReadBoundedNumber(0.0, 1.0, rho);

    CentralFilterVec(phi, theta, sigmaVec, rhoVec, lambda, rho, s); {in RESPONSE}
{WKCoefficients(phi, theta, s - 1, lambda); already in the above}

{in RESPONSE}
    GetFilterGainVector(filterSpec, SCREENBOX, spectraVec, spectraBOX, phi, theta, phi1, theta1, phi2, theta2, sPlus, q);

    if (wehavePolyResiduals or weHaveButterworthResiduals) then
      begin
        plotBOX := dataBOX;
        SlowFourier(residueBOX, residueVec, alpha, beta);
        MakeThePeriodogram(residueBOX, plotBOX, plotVector, alpha, beta);
        MakeDataBox(plotBOX, plotVector);
        plotBOX.yLB := 0;
        plotBOX.xRB := pi;
        plotBOX.xLB := 0;
        plotBOX.yUB := 1.2 * plotBOX.yUB;
        plotBOX.yRange := plotBOX.yUB;
      end;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrameLite(spectraBOX, SCREENBOX);

    if (filterSpec.seasonality = Monthly) then
      DoDecAxis(SCREENBOX)
    else
      ZeroPiAxis(SCREENBOX);

    if (wehavePolyResiduals or weHaveButterworthResiduals) then
      begin   {plot the periodogram}
        SetColor(red);
        SetLineStyle(0, 0, ThickWidth);
        ScreenPlot(plotBOX, SCREENBOX, plotVector);
        SetColor(yellow);
        SetLineStyle(0, 0, NormWidth);
      end;

    ScreenPlotMod(spectraBOX, SCREENBOX, spectraVec);

    legend := SeasonalFilterGain(filterSpec);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

    if PSPict then
      begin
        if wehavePolyResiduals then   {superimpose the periodogram via PlotBOX, plotVector}
          PostResponseOverlay(F, plotBOX, spectraBOX, plotVector, spectraVec, POSTBOX, legend)
        else
          PostSpectraPlot(F, filterSpec, spectraBOX, spectraVec, POSTBOX, legend);
      end;

      noSeaFilterSpec := false;

  end; {OrdinaryFilterSpec}

{_________________________________________________}

  procedure SmoothingFilterSpec (var filterSpec: FilterRecord;
                  var rhoVec, sigmaVec, phi, theta, theta2: seasVector;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX);

    var
      s: integer;
      F: text;
      legend: string;

  begin
    s := filterSpec.seasonality;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}

    ClrScr;

    Writeln;
    Writeln;
    Writeln('   THE SEASONAL-ADJUSTMENT FILTER WITH SMOOTHING');
    Writeln;
    Writeln('   The economic  models that  underlie the SEATS-TRAMO and  STAMP programs');
    Writeln('   contain explicit trend  functions  of a random  walk nature. These give');
    Writeln('   rise to filters that can  be used  to extract tend-cycle functions from');
    Writeln('   the data, which are somewhat smoother than the seasonally adjusted data');
    Writeln('   sequences.');
    Writeln;
    Writeln('   In the seasonal adjustment  procedures of the SEADOS program, the trend');
    Writeln('   is extracted  by a  polynomial regression. The regression residuals are');
    Writeln('   subjected to  a  seasonal adjustment filter, whereafter  they are added');
    Writeln('   back to the polynomial to create the seasonally adjusted data.');
    Writeln;
    Writeln('   The trend  extraction filters of the SEATS-TRAMO and the STAMP programs');
    Writeln('   can be mimicked nevertheless by applying a low pass smoothing filter to');
    Writeln('   the seasonally adjusted residuals. The resulting sequence is similar to');
    Writeln('   those obtained by the above-mentioned programs.');
    Writeln(' ' : 66, '<RETURN>');
    Readln;

    Writeln('    Specify value of the lambda (0.5 works well)');
    ReadBoundedNumber(0.0, 1.0, lambda);
    Writeln('    Specify value of the  rho (0.8 works well)');
    ReadBoundedNumber(0.0, 1.0, rho);

    GetSmoothingParameters(muVec, theta2, q);

{muVec is for the filter to be applied to the data}
{and theta2 is for the frequency response function}
{Should muVec be in the procedural declaration?}

     filterSpec.filterType := taperFunction;

{First, we plot the frequency response function of the smoothing filter}
     filterSpec.filterType := taperFunction;

    GetFilterGainVector(filterSpec, SCREENBOX, spectraVec, spectraBOX, phi, theta, phi1, theta1, phi2, theta2, s, q);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrameLite(spectraBOX, SCREENBOX);
    if (filterSpec.seasonality = Monthly) then
      DoDecAxis(SCREENBOX)
    else
      ZeroPiAxis(SCREENBOX);
    ScreenPlotMod(spectraBOX, SCREENBOX, spectraVec);

    legend := 'The frequency response of the smoothing function';
    ScreenCaption(legend);

{+++++++++++++++++++++++++++++++++++++++++++++++++++++}
   CallForReturn;
   Readln;
   CloseGraph;
   RestoreCrtMode;

   ClrScr;

{Now we do the frequency response function of the Trend-Cycle Filter}

     filterSpec.filterType := taperedFilter;

    CentralFilterVec(phi, theta, sigmaVec, rhoVec, lambda, rho, s); {in RESPONSE}
    GetFilterGainVector(filterSpec, SCREENBOX, spectraVec, spectraBOX, phi, theta, phi1, theta1, phi2, theta2, s, q);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrameLite(spectraBOX, SCREENBOX);
    if filterSpec.seasonality = Monthly then
      DoDecAxis(SCREENBOX)
    else
      ZeroPiAxis(SCREENBOX);
    ScreenPlotMod(spectraBOX, SCREENBOX, spectraVec);

    legend := ('The frequency response of the tapered filter for trend-cycle estimation');

    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

    if PSPict then
      PostSpectraPlot(F, filterSpec, spectraBOX, spectraVec, POSTBOX, legend);

     noSeaFilterSpec := false;

  end;{SmoothingFilterSpec}

{_________________________________________________}

  procedure TwoPassFilterSpec (var filterSpec: FilterRecord;
                  var rhoVec, sigmaVec1, sigmaVec2: seasVector;
                  var phi1, theta1, phi2, theta2: seasVector;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX);

{In this routine, the offsets are symmetric relative to the central seasonal frequency.}
{We could use <OffsetFilterVecs> here.}

    var
{lambda, rho, zeta: real; these are global}
      s, sPlus, quadNo: integer;
      pi: real;
      plotBOX: rBOX;
      plotVector: longVector;
      alpha, beta: fourierVec;
      F: text;
      legend: string;

  begin
    pi := 4.0 * Arctan(1.0);
    filterSpec.filterType := doubleFilter;
    offset := true;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}

    ClrScr;

    Writeln;
    Writeln;
    Writeln('   THE DOUBLE OFFSET FILTER');
    Writeln;
    Writeln('   The  twopass offset seasonal  adjustment filter comprises two filters  that');
    Writeln('   are applied in  sequence. The filters have poles and zeros that are located');
    Writeln('   at small distances below and above the  seasonal frequencies. The intention');
    Writeln('   is to widen the stopands that surround the seasonal frequencies. An adverse');
    Writeln('   effect of this design is to  allow a minor leakage to occur at the seasonal');
    Writeln('   frequencies, which will become more severe as the offsets are increased.');
    Writeln;
    Writeln(' ' : 70, '<RETURN>');
    Readln;

    Writeln('    Specify value of the lambda (0.5 works well)');
    ReadBoundedNumber(0.0, 1.0, lambda);

    Writeln('    Specify value of the  rho (0.8 works well)');
    ReadBoundedNumber(0.0, 1.0, rho);

    Writeln('    The offsets are the same on both sides of the seasonal frequencies.');
    Writeln('    They are the same value, at most 4 degrees, for all the frequencies.');

    Writeln('    Specify value of the  offset zeta  in degrees');
    ReadBoundedNumber(0.0, 4.0, zeta);


    s := filterSpec.s;
    quadNo := (s - 2) div 2; {1 for quarterly, 5 for monthly ?}

{Observe that with the offsets, there is a quadratic Factor (at pi \pm offset)}
{which we choose to exclude from the first filter. i.e from phi1, theta1.}

{First Offset Vec: phi1, theta1}
    sPlus := 2 * quadNo; {sPlus is the degree of the filter polynomials}
    zeta := -zeta;
    if s = 4 then
      QuarterlyQuads(sigmaVec1, rhoVec, rho, zeta, quadNo)
    else
      CompoundQuads(sigmaVec1, rhoVec, rho, zeta, quadNo);
    Laurent(sigmaVec1, theta1, s - 2);
    Laurent(rhoVec, phi1, s - 2);
    WKCoefficients(phi1, theta1, sPlus, lambda);

{Second Offset Vec: phi2, theta2}
    zeta := -zeta;
    quadNo := quadNo + 1;
    sPlus := 2 * quadNo; {sPlus is the degree of the filter polynomials}
    if s = 4 then
      QuarterlyQuads(sigmaVec2, rhoVec, rho, zeta, quadNo)
    else
      CompoundQuads(sigmaVec2, rhoVec, rho, zeta, quadNo);

    Laurent(sigmaVec2, theta2, s);
    Laurent(rhoVec, phi2, s);
    WKCoefficients(phi2, theta2, sPlus, lambda);

{in RESPONSE}
    GetFilterGainVector(filterSpec, SCREENBOX, spectraVec, spectraBOX, phi, theta, phi1, theta1, phi2, theta2, s, q);

    if (wehavePolyResiduals or weHaveButterworthResiduals) then
      begin
        plotBOX := dataBOX;
        SlowFourier(residueBOX, residueVec, alpha, beta);
        MakeThePeriodogram(residueBOX, plotBOX, plotVector, alpha, beta);
        MakeDataBox(plotBOX, plotVector);
        plotBOX.yLB := 0;
        plotBOX.xRB := pi;
        plotBOX.xLB := 0;
        plotBOX.yUB := 1.2 * plotBOX.yUB;
        plotBOX.yRange := plotBOX.yUB;
      end;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrameLite(spectraBOX, SCREENBOX);

    if (filterSpec.seasonality = Monthly) then
      DoDecAxis(SCREENBOX)
    else
      ZeroPiAxis(SCREENBOX);

    if (wehavePolyResiduals or weHaveButterworthResiduals) then
      begin
        SetColor(red);
        SetLineStyle(0, 0, ThickWidth);
        ScreenPlot(plotBOX, SCREENBOX, plotVector);
        SetColor(yellow);
        SetLineStyle(0, 0, NormWidth);
      end;

    ScreenPlotMod(spectraBOX, SCREENBOX, spectraVec);

    legend := DoubleFilterGain(filterSpec);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

    if PSPict then
      begin
        if wehavePolyResiduals then
          PostResponseOverlay(F, plotBOX, spectraBOX, plotVector, spectraVec, POSTBOX, legend)
        else
          PostSpectraPlot(F, filterSpec, spectraBOX, spectraVec, POSTBOX, legend);
      end;

     noSeaFilterSpec := false;

  end; {TwoPassFilterSpec}

{_________________________________________________}

  procedure GetTheOffsets (var filterSpec: FilterRecord;
                  var zetaSub, zetaSup: vector;
                  var quadNo: integer);
    var
      i, s, limit: integer;
      seasFreak: integer;
      STR1, addString: string;
      degreesBelow, degreesAbove: integer;

  begin
    s := filterSpec.s;
    quadNo := s div 2; {6 or 2}

    if s = 12 then
      seasFreak := 30;
    if s = 4 then
      seasFreak := 90;

    degreesBelow := 0;
    degreesAbove := 0;

    limit := 180 div s;

    for i := 1 to quadNo do
      begin {i}
        Writeln('    Specify the  additional poles and zeros adjacent to the seasonal ');
        addString := IntegerToString(seasFreak);
        STR1 := Concat('    frequency at ', addString, ' degrees');
        Writeln(STR1);

        Writeln;
        Writeln('    Specify the number of degrees below the seasonal frequency');
        GetBoundedInteger(0, limit, degreesBelow);
        if degreesBelow > 0 then
          zetaSub[i] := -degreesBelow
        else
          zetaSub[i] := 0.0;

       if i = quadNo then
         zetaSup[i] := zetaSub[i];

       if (seasFreak < 180) then
         begin
           Writeln('    Specify the number of degrees above the seasonal frequency');
           GetBoundedInteger(0, limit, degreesAbove);
           if degreesAbove > 0 then
             zetaSup[i] := degreesAbove
           else
            zetaSup[i] := 0.0;
         end; {seasFreak < 180}

        if s = 12 then
          seasFreak := seasFreak + 30;
        if s = 4 then
          seasFreak := seasFreak + 90;
      end; {i}

  end; {GetTheOffsets}

{_________________________________________________}

  procedure OffsetFilterVecs (var filterSpec: FilterRecord;
                  var sigmaVec1, sigmaVec2: seasVector;
                  var phi1, theta1, phi2, theta2: seasVector;
                  var lambda, rho: real;
                  var quadNo: integer);

{This routine serves only for the triple filter}
    var
      zetaSub, zetaSup: vector;
{sigmaVec, rhoVec: seasVector; THESE ARE GLOBAL}
      s, sPlus: integer;

  begin
    s := filterSpec.s;
    quadNo := s div 2;
    sPlus := 2 * quadNo;

    GetTheOffsets(filterSpec, zetaSub, zetaSup, quadNo);

    quadNo := (s - 2) div 2;
    sPlus := 2 * quadNo; {twice quadNo }

    if s = 12 then
      NewCompoundQuads(sigmaVec1, rhoVec, zetaSub, rho, quadNo);
    if s = 4 then
      NewQuarterlyQuads(sigmaVec1, rhoVec, zetaSub, rho, quadNo);

    Laurent(sigmaVec1, theta1, s - 2); {Theta1 is 10 of 1}
    Laurent(rhoVec, phi1, s - 2);
    WKCoefficients(phi1, theta1, sPlus, lambda);

    quadNo := s div 2;
    sPlus := 2 * quadNo;

    if s = 12 then
      NewCompoundQuads(sigmaVec2, rhoVec, zetaSup, rho, quadNo);
    if s = 4 then
      NewQuarterlyQuads(sigmaVec2, rhoVec, zetaSup, rho, quadNo);

    Laurent(sigmaVec2, theta2, s);
    Laurent(rhoVec, phi2, s);
    WKCoefficients(phi2, theta2, s, lambda);

  end; {OffsetFilterVecs }

{_________________________________________________}

  procedure TripleFilterSpec (var filterSpec: FilterRecord;
                  var sigmaVec, sigmaVec1, sigmaVec2: seasVector;
                  var phi, theta, phi1, theta1, phi2, theta2: seasVector;
                  var spectraVec: longvector;
                  var spectraBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      quadNo, s: integer; {t is TEMP }
      pi: real;
      alpha, beta: fourierVec;
      plotBOX: rBOX;
      plotVector: longvector;
      legend: string;
      F: text;

  begin
    pi := 4.0 * Arctan(1.0);
     filterSpec.filterType := tripleFilter;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}

    ClrScr;

    Writeln;
    Writeln;
    Writeln('    THE TRIPLE FILTER WITH OFFSETS');
    Writeln;
    Writeln('    The triple filter supplements the ordinary  seasonal adjustment filter by');
    Writeln('    two  offset filters that  have  their pole-zero pairs below and above the');
    Writeln('    seasonal frequencies. The user is asked to specify the offsets in mumbers');
    Writeln('    of degrees.');
    Writeln;
    Writeln(' ' : 69, '<RETURN>');
    Readln;

    Writeln('    Specify value of the lambda (0.5 works well)');
    ReadBoundedNumber(0.0, 1.0, lambda);

    Writeln('    Specify value of the  rho (0.8 works well)');
    ReadBoundedNumber(0.0, 1.0, rho);

    s := filterSpec.s;
    quadNo := (s - 2) div 2; {This will serve for central filter}

    offset := true;
    CentralFilterVec(phi, theta, sigmaVec, rhoVec, lambda, rho, s); {in RESPONSE}
{sigmaVec and rhoVec could be local}

{This generates both offset filters}
    offset := true;
    OffsetFilterVecs(filterSpec, sigmaVec1, sigmaVec2, phi1, theta1, phi2, theta2, lambda, rho, quadNo); {in ORGANISE}

{in RESPONSE}
    GetFilterGainVector(filterSpec, SCREENBOX, spectraVec, spectraBOX, phi, theta, phi1, theta1, phi2, theta2, s, q);

    if (wehavePolyResiduals or weHaveButterworthResiduals) then
      begin
        plotBOX := dataBOX;
        SlowFourier(residueBOX, residueVec, alpha, beta);
        MakeThePeriodogram(residueBOX, plotBOX, plotVector, alpha, beta);
        MakeDataBox(plotBOX, plotVector);
        plotBOX.yLB := 0;
        plotBOX.xRB := pi;
        plotBOX.xLB := 0;
        plotBOX.yUB := 1.2 * plotBOX.yUB;
        plotBOX.yRange := plotBOX.yUB;
      end;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrameLite(spectraBOX, SCREENBOX);

    if (filterSpec.seasonality = Monthly) then
      DoDecAxis(SCREENBOX)
    else
      ZeroPiAxis(SCREENBOX);

    if (wehavePolyResiduals or weHaveButterworthResiduals) then
      begin
        SetColor(red);
        SetLineStyle(0, 0, ThickWidth);
        ScreenPlot(plotBOX, SCREENBOX, plotVector);
        SetColor(yellow);
        SetLineStyle(0, 0, NormWidth);
      end;

    ScreenPlotMod(spectraBOX, SCREENBOX, spectraVec);

    legend := TripleFilterGain(filterSpec);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

    if PSPict then
      begin
        if wehavePolyResiduals then
          PostResponseOverlay(F, plotBOX, spectraBOX, plotVector, spectraVec, POSTBOX, legend)
        else
          PostSpectraPlot(F, filterSpec, spectraBOX, spectraVec, POSTBOX, legend);
      end;
     noSeaFilterSpec := false;

  end; {TripleFilterSpec}
  {_________________________________________________}

    procedure ButterworthSpec (var filterSpec: FilterRecord;
                    var spectraBOX: rBOX;
                    var spectraVec: longVector;
                    var SCREENBOX, POSTBOX: IBOX);

  {ButterworthResponse --> ButterworthSpec}
      var
        s: integer;
        pi: real;
        alpha, beta: fourierVec;
        plotBOX: rBOX;
        plotVector: longvector;
        legend: string;
        F: text;

    begin
      s := 0;
      pi := 4.0 * Arctan(1.0);

      FilterSpec.filterType := Butterworth;

      ClrScr;

      Writeln;
      Writeln('    SPECIFY THE LOWPASS BUTTERWORTH FILTER');
      Writeln;
      Writeln('    The lowpass Butterworth time-domain filter, which  operates  directly on');
      Writeln('    the data, which may exhibit a trend, comprises two parameters. The first');
      Writeln('    of  these is the nominal  cutoff point, which  is  the mid point  in the');
      Writeln('    transition from the stop band  to the pass band. The second parameter is');
      Writeln('    the order of the filter, which is an integer value. Increasing the order');
      Writeln('    makes the transition more rapid.');
      Writeln;
      Writeln('    The nominal cutoff point can be determined in view of the periodogram of');
      Writeln('    the  residual deviations of the data from a polynomial interpolated by a');
      Writeln('    least-squares regression.');
      Writeln;
      Writeln(' ' : 68, '<RETURN>');
      Readln;

      with filterSpec do
        begin {filterSpec}

          Writeln('    Specify the cut-off of the lowpass digital filter');
          Writeln('    as a number of degrees in the interval (0, 180)');
          ReadBoundedNumber(0.0, 180.0, cutOff);

          Writeln;

          Writeln('    What is the order of the unidirectional filter?');
          GetBoundedInteger(0, 15, filterOrder);


          cutOff := cutOff * pi / 180.0;
        end; {FilterRecord}

  {in RESPONSE}
      GetFilterGainVector(filterSpec, SCREENBOX, spectraVec, spectraBOX, phi, theta, phi1, theta1, phi2, theta2, s, q);

      if wehavePolyResiduals then
         begin
           plotBOX := dataBOX;
           SlowFourier(residueBOX, residueVec, alpha, beta);
           MakeThePeriodogram(residueBOX, plotBOX, plotVector, alpha, beta);
           MakeDataBox(plotBOX, plotVector);
           plotBOX.yLB := 0;
           plotBOX.xRB := pi;
           plotBOX.xLB := 0;
           plotBOX.yUB := 1.2 * plotBOX.yUB;
           legend := ButterworthOverlay;
           plotBOX.yRange := plotBOX.yUB;
         end;

         legend := GainOfButterworth(filterSpec);

      GetGraphics(graphicsOn, SCREENBOX, currentMode);

      ScreenFrameLite(spectraBOX, SCREENBOX);

      if (filterSpec.seasonality = Monthly) then
        DoDecAxis(SCREENBOX)
      else
        ZeroPiAxis(SCREENBOX);

      if wehavePolyResiduals then
        begin
          SetColor(red);
          SetLineStyle(0, 0, ThickWidth);
          ScreenPlot(plotBOX, SCREENBOX, plotVector);
          SetColor(yellow);
          SetLineStyle(0, 0, NormWidth);
        end;

      ScreenPlotMod(spectraBOX, SCREENBOX, spectraVec);

      ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;


    if PSPict then
      begin
        if wehavePolyResiduals then
          PostResponseOverlay(F, plotBOX, spectraBOX, plotVector, spectraVec, POSTBOX, legend)
        else
          PostSpectraPlot(F, filterSpec, spectraBOX, spectraVec, POSTBOX, legend);
      end;

      noButterworthSpec := false;

    end; {ButterworthSpec}

  {_________________________________________________}

end. {RESPONSE unit 8 of SEASIDE.PAS}

