unit SCREEN; {Unit 6 of SEASIDE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, Dos, Crt, Graph;

  procedure MarksAndNumbers (XSTART, YSTART: integer;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

  procedure ScreenFrame (var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure ScreenFrameLite (var realBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure VertAxis (XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

  procedure ZeroPiAxis (var SCREENBOX: IBOX);

  procedure DoDecAxis (var SCREENBOX: IBOX);

  procedure ScreenPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

  procedure ScreenPlotMod (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

  procedure ScreenOverplot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector, trendVector: longVector;
                  var legend: string);


  procedure ScreenMakePlot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);

implementation

{________________________________________________}

  procedure MarksAndNumbers (XSTART, YSTART: integer;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

    var
      toly, HRL, VRL: real;
      HR1, HR2, VT1, VT2, tolyWidth: integer;
      HINC, VINC, PENLEFT, PENDOWN: integer;
      tolyString: string;

  begin {MarksAndNumbers}

    if Axis = 'yScale' then
      begin {Pixel Parameters}
        HINC := 5;
        VINC := 0;
        PENDOWN := -(TextHeight('NUMBERS') div 2);
      end;
    if Axis = 'xScale' then
      begin
        HINC := 0;
        VINC := -5;
        PENDOWN := 20;
      end; {Pixel Parameters}

    toly := firstMark;
    HRL := XSTART;
    HR1 := XSTART;
    HR2 := HR1 + HINC;
    VRL := YSTART;
    VT1 := YSTART;
    VT2 := VT1 + VINC;

    while Abs(toly) <= Abs(farBound) do

      begin {while toly}
        tolyString := TalyToString(toly);
        tolyWidth := TextWidth(tolyString);
        if axis = 'xScale' then
          PENLEFT := Round(tolyWidth / 2);
        if axis = 'yScale' then
          PENLEFT := tolyWidth + 15;

 {Numbers}
        MoveTo(HR1 - PENLEFT, VT1 + PENDOWN);
        OutText(tolyString);

  {Hatchmarks}
        MoveTo(HR1, VT1);
        LineTo(HR2, VT2);

  {Increments}
        if axis = 'xScale' then
          HRL := HRL + STEP;
        HR1 := Round(HRL);
        HR2 := HR1 + HINC;
        if axis = 'yScale' then
          VRL := VRL - STEP;
        VT1 := Round(VRL);
        VT2 := VT1 + VINC;

        toly := toly + taly;
      end; {while toly}

  end; {MarksAndNumbers}

{________________________________________________}

  procedure ScreenFrame (var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

    {This is the revised version of ScreenFrame}

    {Uses, MarksAndNumbers, TallyPoints, TalyToString}
    {Depends on prior application of InitialiseDataBOX }
    {and InitialiseAnIBOX}

    var
      FIRSTHATCH, XORIGIN, YORIGIN, YCap, sign: integer;
      rxPos, rxNeg, ryPos, ryNeg, vSTEP, hSTEP, taly: real;
      nearBound, farBound, boundRange, increment, startMark: real;

  begin
        {ShowMessage('We are in ScreenFrame');}
    with SCREENBOX, dataBOX do
      begin
        yUB := yMax;
        yLB := yMin;

        yRange := yMax - yMin;
        xRange := xMax - xMin;

    {(X1,  Y1) is the top left of the IBOX}
    {(X2,  Y2) is the botton lright of the IBOX}

        XORIGIN := X1;{ Default for  XORIGIN}
        YORIGIN := Y2;{ Default for  YORIGIN}

        if doFrame then
          Rectangle(X1, Y1, X2 + 1, Y2 + 1);

        if yScale = true then
          begin {Do the yScale}

    {...............................................................................................}

            if yMin * yMax >= 0 then
              begin {yMin, yMax > 0 or yMin, yMax < 0}

                if yMax < 0 then
                  begin
                    sign := -1;
                    nearBound := yMax;
                    farBound := yMin;
                  end
                else
                  begin
                    sign := 1;
                    nearBound := yMin;
                    farBound := yMax
                  end;

                if ((farBound - nearBound) / farBound) < 0.3 then
                  begin {small boundRange}

                    nearBound := nearBound - sign * (0.2 * yRange);
                    farBound := farBound + sign * (0.2 * yRange);
                    if sign = 1 then
                      begin
                        yUB := farBound;
                        yLB := nearBound
                      end
                    else
                      begin
                        yLB := farBound;
                        yUB := nearBound
                      end;

                    boundRange := Abs(farBound - nearBound);

                    TallyPoints(boundRange, taly);
                    startMark := Trunc(nearBound / taly) * taly;
                    if (Abs((startMark - nearBound) / nearBound) > 0.0001) then
                      startMark := startMark + sign * taly;

                    if sign = 1 then
                      Ycap := Y2
                    else
                      Ycap := Y1;

                    FIRSTHATCH := Ycap + Round((nearBound - startMark) * BOXHGT / boundRange);
                  end {small boundRange}

                else{large boundRange}
                  begin {yMin = 0.0}

                    if yMax > 0 then
                      begin
                        farBound := 1.2 * yMax;
                        yUB := farBound;
                        yLB := 0.0;
                      end
                    else
                      begin
                        farBound := 1.2 * yMin;
                        yUB := 0.0;
                        yLB := farbound;
                      end;

                    if sign = 1 then
                      Ycap := Y2
                    else
                      Ycap := Y1;

                    boundRange := Abs(farBound);
                    TallyPoints(boundRange, taly);
                    startMark := 0.0;
                    FIRSTHATCH := Ycap;

                  end; {large boundRange}

                vSTEP := taly * (BOXHGT / boundRange);
                hSTEP := 0.0;
                MarksAndNumbers(X1, FIRSTHATCH, sign * taly, startMark, farBound, sign * vSTEP, 'yScale');

              end;{yMin, yMax > 0 or yMin, yMax < 0}
    {...............................................................................................}
               {if ymax * ymin < 0}
            if (yMin < 0.0) and (yMax > 0.0) then
              begin {yMin < 0.0}
                ryPos := yMax;
                ryNeg := yMin;

                increment := 0.1 * (yMax - yMin);
                if yMax > Abs(yMin) then
                  begin
                    ryPos := 1.2 * yMax;
                    ryNeg := yMin - increment;
                  end
                else
                  begin
                    ryNeg := 1.2 * yMin;
                    ryPos := yMax + increment
                  end;
                yUB := ryPos;
                yLB := ryNeg;

                YORIGIN := Y2 + Round(BOXHGT * ryNeg / (ryPos - ryNeg));   {ryNeg / (yUB - yLB)}

                if (xScale = true) or (xAxis = true) then
                  begin {this draws the horizontal axis through y = 0}
                    MoveTo(X1, YORIGIN);
                    LineTo(X2, YORIGIN)
                  end;

                if ryPos >= Abs(ryNeg) then
                  begin
                    TallyPoints(ryPos, taly);
                    vSTEP := (YORIGIN - Y1) * (taly / ryPos);
                  end
                else {if ryPos < Abs(ryNeg)}
                  begin
                    TallyPoints(-ryNeg, taly);
                    vSTEP := (YORIGIN - Y2) * (taly / ryNeg);
                  end;

                hSTEP := 0.0;
                MarksAndNumbers(X1, YORIGIN, taly, 0.0, ryPos, vSTEP, 'yScale');
                MarksAndNumbers(X1, YORIGIN, -taly, 0.0, ryNeg, -vSTEP, 'yScale');

              end; {yMin < 0.0 and yMax > 0.0}
    {...............................................................................................}

          end; {Do the yScale}

    {-----------------------------------------------}

        if xScale then
          begin {Do the xScale}

    {...............................................................................................}

            if (xMin <= 0.0) and (xMax >= 0) then
              begin {xMin <= 0.0}
                rxPos := xMax;
                rxNeg := xMin;

                if (xMin < 0.0) then
                  XORIGIN := X1 - Round(BOXWDT * rxNeg / (rxPos - rxNeg))
                else
                  XORIGIN := X1;

                if (xMin < 0.0) and (yScale = true) then
                  begin  {This provides the vertical axis through x =0}
                    MoveTo(XORIGIN, Y1);
                    LineTo(XORIGIN, Y2)
                  end;

                if rxPos > Abs(rxNeg) then
                  begin
                    TallyPoints(rxPos, taly);
                    hSTEP := (X2 - XORIGIN) * (taly / rxPos);
                  end
                else
                  begin
                    TallyPoints(-rxNeg, taly);
                    hSTEP := (X1 - XORIGIN) * (taly / rxNeg);
                  end;

                VSTEP := 0;
                if xMax > 0.0 then
                  MarksAndNumbers(XORIGIN, Y2, taly, 0, rxPos, hSTEP, 'xScale');
                if xMin < 0.0 then
                  MarksAndNumbers(XORIGIN, Y2, -taly, 0, rxNeg, -hSTEP, 'xScale');
              end; {xMin <= 0.0}

    {...............................................................................................}

            if xMin * xMax > 0.0 then
              begin {xMin, xMax < 0 or xMin, xMax >0}

                sign := 1;
                if xMax < 0 then
                  sign := -1;

                TallyPoints(xRange, taly);
                if xMin > 0 then
                  begin
                    farBound := xMax;
                    nearBound := xMin;
                  end;
                if xMax < 0 then
                  begin
                    farBound := xMin;
                    nearBound := xMax;
                  end;

                startMark := Trunc(nearBound / taly) * taly;
                if (Abs((startMark - nearBound) / nearBound) > 0.0001) then
                  startMark := startMark + sign * taly;
                hSTEP := (BOXWDT) * (taly / xRange);
                vSTEP := 0.0;
                if sign = 1 then
                  FIRSTHATCH := X1 + Round((startMark - xMin) * BOXWDT / xRange)
                else
                  FIRSTHATCH := X2 - Round((xMax - startMark) * BOXWDT / xRange);
                MarksAndNumbers(FIRSTHATCH, Y2, sign * taly, startMark, farBound, sign * hSTEP, 'xScale');
              end;
    {...............................................................................................}

          end; {Do the xScale}

    {-----------------------------------------------}
        X0 := XORIGIN;
        Y0 := YORIGIN;

      end; {with SCREENBOX and dataBOX}

  end; {ScreenFrame}

{________________________________________________}

  procedure ScreenFrameLite (var realBOX: rBOX;
                  var SCREENBOX: IBOX);

    var
      BOXHGT: integer;
      XSTART: integer;
      VSTEP, taly: real;
      yMax, yMin: real;
      lowerBound, upperBound: real;

  begin
    yMax := realBOX.yMax; {the maximum value of y in the Data vector}
    yMin := realBOX.yMin; {the minimum value of y in the Data vector}

    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1; {Height of Bounding Box}

    Rectangle(SCREENBOX.X1, SCREENBOX.Y1, SCREENBOX.X2 + 1, SCREENBOX.Y2 + 1);

    lowerBound := 0.0;
    upperBound := 1.2 * yMax;
    Tallypoints(upperBound, taly);

    XSTART := SCREENBOX.X1;
    VSTEP := taly * (BOXHGT / upperBound);
    VertAxis(XSTART, SCREENBOX.Y2, taly, lowerBound, upperBound, VSTEP);

    realBOX.yUB := upperBound;
    realBOX.yLB := lowerBound;
    realBOX.xLB := realBOX.xMin;
    realBOX.xRB := realBOX.xMax;

    SCREENBOX.X0 := SCREENBOX.X1;
    SCREENBOX.Y0 := SCREENBOX.Y2;

  end; {ScreenFrameLite}

{_________________________________________________}

  procedure VertAxis (XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

    var
      toly, HRL, VRL: real;
      HR1, HR2, VT, tolyWidth: integer;
      HINC, VINC, PENLEFT, PENDOWN: integer;
      tolyString: string;

  begin {VertAxis}

    HINC := 5;
    PENDOWN := 4;

    toly := firstMark;
    HRL := XSTART;
    HR1 := XSTART;
    HR2 := HR1 + HINC;
    VRL := YSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do
      begin {while toly}

        tolyString := TalyToString(toly);
        tolyWidth := TextWidth(tolyString);
        PENLEFT := tolyWidth + 15;

 {Numbers}
        MoveTo(HR1 - PENLEFT, VT + PENDOWN);
        OutText(tolyString);

  {Hatchmarks}
        MoveTo(HR1, VT);
        LineTo(HR2, VT);

  {Increments}
        VRL := VRL - VSTEP;
        VT := Round(VRL);

        toly := toly + taly;
      end; {while toly}

  end; {VertAxis}

{_________________________________________________}

  procedure ZeroPiAxis (var SCREENBOX: IBOX);

    var
      HR, VT1, VT2, BOXWDT, PENLEFT, PENDOWN: integer;

  begin{ZeroPiAxis}
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    PENLEFT :=  12;
    PENDOWN := 20;

    VT1 := SCREENBOX.Y2;
    VT2 := VT1 - 5;
    HR := SCREENBOX.X1;
    MoveTo(HR - 2, VT1 + PENDOWN);
    OutText('0');

    HR := SCREENBOX.X1 + Round(BOXWDT / 4);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/4');

    HR := SCREENBOX.X1 + Round(BOXWDT / 2);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/2');

    HR := SCREENBOX.X2 - Round(BOXWDT / 4);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - PENLEFT, VT1 + PENDOWN);
    OutText('3p/4');

    HR := SCREENBOX.X2;
    MoveTo(HR - 2, VT1 + PENDOWN);
    OutText('p');

  end;{ZeroPiAxis}

{_________________________________________________}

  procedure DoDecAxis (var SCREENBOX: IBOX);

    var
      HR, VT1, VT2, BOXWDT, PENDOWN: integer;

  begin{DoDecAxis}
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    PENDOWN := 20;

    VT1 := SCREENBOX.Y2;
    VT2 := VT1 - 5;
    HR := SCREENBOX.X1;
    MoveTo(HR - 2, VT1 + PENDOWN);

    HR := SCREENBOX.X1 + Round(BOXWDT / 6);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/6');

    HR := SCREENBOX.X1 + Round(BOXWDT / 3.0);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/3');

    HR := SCREENBOX.X2 - Round(BOXWDT / 2);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/2');

    HR := SCREENBOX.X2 - Round(BOXWDT / 3.0);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 12, VT1 + PENDOWN);
    OutText('2p/3');

    HR := SCREENBOX.X2 - Round(BOXWDT / 6.0);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 12, VT1 + PENDOWN);
    OutText('5p/6');

    HR := SCREENBOX.X2;
    MoveTo(HR - 2, VT1 + PENDOWN);
    OutText('p');

  end;{DoDecAxis}

{_________________________________________________}

  procedure ScreenPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting an arbitrary number of data points over}
{the range defined by the width of the bounding box}

    var
      i, HR, VT, BOXHGT: integer;
      xRange, yRange, BOXWDT: real;
      store: double;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := realBOX.yUB - realBOX.yLB;
    xRange := realBOX.xRB - realBOX.xLB;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
        store := i * BOXWDT;
        store := store / (realBOX.NOPUNCT - 1);
        HR := SCREENBOX.X1 + Round(store);

        store := (dataVector[i] - realBOX.yLB) / yRange;
        VT := Round(SCREENBOX.Y2 - store * BOXHGT);

        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}

  end; {ScreenPlot}

{_________________________________________________}

  procedure ScreenPlotMod (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting a series of function evaluations thar are equal}
{in number to the width to the bounding box measured in pixelpoints.}

    var
      i, HR, VT, XORIGIN, YORIGIN, BOXWDT, BOXHGT: integer;
      store, yRange: real;

  begin
    yRange := realBOX.yUB - realBOX.yLB;
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    YORIGIN := Round(SCREENBOX.Y0);
    XORIGIN := Round(SCREENBOX.X0);

    for i := 0 to BOXWDT do
      begin {i}
        HR := XORIGIN + i;
        store := dataVector[i] * (BOXHGT / yRange);
        VT := YORIGIN - Round(store);
        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}
  end; {ScreenPlotMod}

{________________________________________________}

  procedure ScreenOverplot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector, trendVector: longVector;
                  var legend: string);

  begin
    ClrScr;

    Writeln;
    Writeln(' ' : 60, '<RETURN>');

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);

    SetColor(red);
    SetLineStyle(0, 0, ThickWidth);
    ScreenPlot(dataBOX, SCREENBOX, trendVector);
    SetColor(yellow);
    SetLineStyle(0, 0, NormWidth);

    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

  end; {ScreenOverplot}

{_________________________________________________}

  procedure ScreenMakePlot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);

  begin
    ClrScr;

    Writeln;
    Writeln(' ' : 60, '<RETURN>');

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;

    ClrScr;

  end; {ScreenMakePlot}

{________________________________________________}


end. {SCREEN: Unit 6 of SEASIDE.PAS}

