program SEADOS;

  uses
    GLOBALS, UTILS, DATASEG, ORGANISE, Dos, Crt, Graph;

  var
    theMenuItem: integer;

{_________________________________________________}

begin
  SetTheText;

  Writeln;

  Writeln('      SEADOS.PAS (D.S.G. Pollock, <stephen_pollock@sigmapi.u-net.com>)');
  Writeln;
  Writeln('      This program implements a variety of time-domain procedures for the');
  Writeln('      seasonal adjustment of economic  data. The Wiener-Kolmogorov method');
  Writeln('      is used in designing filters for removing the seasonal fluctuations');
  Writeln('      from the data.');
  Writeln;
  Writeln('      To adjust a trended data  sequence, one  must begin by extracting a');
  Writeln('      polynomial trend. Then, the seasonal fluctuations  are removed from');
  Writeln('      the residual sequence,  whereafter the result can  be added back to');
  Writeln('      the polynomial trend.');
  Writeln;
  Writeln('      The parameters of the filter  should  be  determined in view of the');
  Writeln('      spectral properties of the detrended  data that are revealed by the');
  Writeln('      periodogram. Elements  with frequencies  adjacent  to  the seasonal');
  Writeln('      frequency and its  harmonics can  be removed by applying additional');
  Writeln('      offset filters in series with the principal central filter.');
  Writeln;
  Writeln(' ' : 65, '<RETURN>');
  Readln;

  {Global Flags}
  graphicsOn := false;
  noSCREENBOX := true;
  SCREENBOX.XC := 0;
  SCREENBOX.YC := 0;

  allsDone:= false;
  thereIsData := false;
  logsTaken := false;
  noPageParameters := true;
  noDataFrequency := true;
  noModel := true;
  noSeaFilterSpec := true;
  noButterworthSpec := true;
  thereisData := false;
  weHaveTrendResiduals := false;
  wehavePolyResiduals := false;
  weHaveButterworthResiduals := false;
  weHaveSeasonalComponent := false;
  weHaveTrendComponent := false;
  weHaveTrendCycleComponent := false;
  weHaveAdjustedResiduals := false;
  dataAreTrended := true;

  dataName := 'an unnamed series';

  repeat
    ClrScr;

    Writeln;
    Writeln;
    Writeln;
    Writeln('    SEADOS.PAS: Enhanced Seasonal Adjustment');
    Writeln;
    Writeln('         1. Page Parameters');
    Writeln;
    Writeln('         2. Get the Data, Transform the Data');
    Writeln;
    Writeln('         3. Plot the Data');
    Writeln;
    Writeln('         4. Detrend the Data');
    Writeln;
    Writeln('         5. Specify the Seasonal Adjustment Filter');
    Writeln;
    Writeln('         6. Do Seasonal Adjustment');
    Writeln;
    Writeln('         7. Save and Exit');
    Writeln;
    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, 7, theMenuItem);

    case theMenuItem of
      1: {in}
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}
      2: {in ORGANISE }
        GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}
      3:
        PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in DATASEG}
      4: {in ORGANISE }
        DetrendTheData(dataBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, residueVec);
      5:  {in ORGANISE}
        SpecifyTheFilter(filterSpec, spectraVec, spectraBOX, rhoVec, phi, phi1, phi2, theta, theta1, theta2);
      6: {in ORGANISE}
        DoSeasonalAdjustment(dataBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, residueVec, polyParas);
      7:
        SaveAndExit(dataBOX, residueBOX, seasonBOX, dataVector, residueVec, seasonVec, adjustedVec);
    end;{case}

  until allsDone;

  RevertText;

end. {main Unit of SEASIDE.PAS}

{_________________________________________________}

  {Compile the Units in the Following Order:}

  {1. GLOBALS}
  {2. MATHS}
  {3. UTILS}
  {4. LEGENDS}
  {5. ROOTS}
  {6. SCREEN}
  {7. POST}
  {8. DATASEG}
  {9. FILTER }
  {10. RESPONSE}
  {11. ORGANISE}
  {12. SEASIDE}

